@@ -112,16 +112,16 @@ sub create_wheel
 {
 	my ($id, $pix, $color)   = @_;
 	my $imul = 256 * $pix / $id;
+	my $i = Prima::DeviceBitmap-> create(
+		width  => 256 * $pix,
+		height => 256 * $pix,
+		name => '',
+	);
 
 	my ( $y1, $x1) = ($id,$id);
 	my  $d0 = $id / 2;
 
-	my $i = Prima::Image->create(
-		width  => $id,
-		height => $id,
-		type   => im::RGB,
-	);
-
+	$i-> begin_paint;
 	$i-> color( cl::Black);
 	$i-> bar( 0, 0, $i-> width, $i-> height);
 
@@ -129,39 +129,44 @@ sub create_wheel
 
 	for ( $y = 0; $y < $y1; $y++) {
 		for ( $x = 0; $x < $x1; $x++) {
-			my ( $h, $s) = xy2hs( $x, $y, $d0);
+			my ( $h, $s, $ok) = xy2hs( $x, $y, $d0);
+			next if $ok;
 			my ( $r, $g, $b) = hsv2rgb( $h, $s, 1);
-			$i-> pixel( $x, $y, $b | ($g << 8) | ($r << 16));
+			$i-> color( $b | ($g << 8) | ($r << 16));
+			$i-> bar(
+				$x * $imul, $y * $imul,
+				( $x + 1) * $imul - 1, ( $y + 1) * $imul - 1
+			);
 		}
 	}
-	$i-> scaling( ist::Gaussian ) if $::application->get_bpp > 8;
-	$i-> size( 256 * $pix, 256 * $pix);
+	$i-> end_paint;
 
-	my $xa = Prima::DeviceBitmap-> create(
+
+	my $a = Prima::DeviceBitmap-> create(
 		width  => 256 * $pix,
 		height => 256 * $pix,
 		name   => 'ColorWheel',
 	);
 
-	$xa-> begin_paint;
-	$xa-> color( $color);
-	$xa-> bar( 0, 0, $xa-> size);
-	$xa-> rop( rop::XorPut);
-	$xa-> put_image( 0, 0, $i);
-	$xa-> rop( rop::CopyPut);
-	$xa-> color( cl::Black);
-	$xa-> fill_ellipse(
+	$a-> begin_paint;
+	$a-> color( $color);
+	$a-> bar( 0, 0, $a-> size);
+	$a-> rop( rop::XorPut);
+	$a-> put_image( 0, 0, $i);
+	$a-> rop( rop::CopyPut);
+	$a-> color( cl::Black);
+	$a-> fill_ellipse(
 		128 * $pix, 128 * $pix,
 		(256 * $pix) - $imul * 2 - 1,
 		(256 * $pix) - $imul * 2 - 1
 	);
-	$xa-> rop( rop::XorPut);
-	$xa-> put_image( 0, 0, $i);
-	$xa-> end_paint;
+	$a-> rop( rop::XorPut);
+	$a-> put_image( 0, 0, $i);
+	$a-> end_paint;
 
 	$i-> destroy;
 
-	return $xa;
+	return $a;
 }
 
 sub create_wheel_shape
@@ -198,7 +203,6 @@ sub profile_default
 		text          => 'Select color',
 
 		quality       => 0,
-		grayscale     => 0,
 		value         => cl::White,
 	}
 }
@@ -355,7 +359,6 @@ sub init
 	$self-> quality( $profile{quality});
 
 	$self-> Roller_Repaint if $self-> {quality};
-	$self-> grayscale($profile{grayscale});
 	return %profile;
 }
 
@@ -507,23 +510,16 @@ sub Roller_Paint
 	$canvas-> clear;
 	my $i;
 	my $step = 8 * $owner->{scaling};
-	my ( $h, $s, $v, $d, $dd) = ( $owner-> {H}-> value, $owner-> {S}-> value,
-		$owner-> {V}-> value, ($size[1] - $step * 2) / 32, ($size[1] - $step * 2) / 256 );
+	my ( $h, $s, $v, $d) = ( $owner-> {H}-> value, $owner-> {S}-> value,
+		$owner-> {V}-> value, ($size[1] - $step * 2) / 32);
 	$s /= 255;
 	$v /= 255;
 	my ( $r, $g, $b);
 
-	my $rlim;
-	if ( $::application->get_bpp > 8 ) {
-		$rlim = 256;
-	} else {
-		$rlim = 32;
-		$dd = $d;
-	}
-	for $i (0..$rlim-1) {
-		( $r, $g, $b) = hsv2rgb( $h, $s, $i / $rlim);
+	for $i (0..31) {
+		( $r, $g, $b) = hsv2rgb( $h, $s, $i / 31);
 		$canvas-> color( cl::from_rgb( $r, $g, $b));
-		$canvas-> bar( $step, $step + $i * $dd, $size[0] - $step, $step + ($i + 1) * $dd);
+		$canvas-> bar( $step, $step + $i * $d, $size[0] - $step, $step + ($i + 1) * $d);
 	}
 
 	$canvas-> color( cl::Black);
@@ -637,16 +633,6 @@ sub set_value
 sub value        {($#_)?$_[0]-> set_value        ($_[1]):return $_[0]-> {value};}
 sub quality      {($#_)?$_[0]-> set_quality      ($_[1]):return $_[0]-> {quality};}
 
-sub grayscale
-{
-	return $_[0]->{grayscale} unless $#_;
-	my ( $self, $gs ) = @_;
-	$self-> {$_}-> enabled(!$gs) for qw(H S R G B wheel);
-	if ( $gs ) {
-		$self-> {$_}-> value(0) for qw(H S);
-	}
-}
-
 package Prima::ColorComboBox;
 use vars qw(@ISA);
 @ISA = qw(Prima::ComboBox);
@@ -674,7 +660,6 @@ sub profile_default
 		width            => 56 * $scaling,
 		literal          => 0,
 		colors           => 20 + 128,
-		grayscale        => 0,
 		editClass        => 'Prima::Widget',
 		listClass        => 'Prima::Widget',
 		editProfile      => {
@@ -702,12 +687,11 @@ sub init
 	$self-> {value} = $profile{value};
 	$self-> {colors} = $profile{colors};
 	@{$profile{listDelegations}} = grep { $_ ne 'SelectItem' } @{$profile{listDelegations}};
-	push ( @{$profile{listDelegations}}, qw(Create Paint MouseDown MouseMove MouseLeave));
-	push ( @{$profile{editDelegations}}, qw(Paint MouseDown Enter Leave Enable Disable KeyDown MouseEnter MouseLeave));
+	push ( @{$profile{listDelegations}}, qw(Create Paint MouseDown));
+	push ( @{$profile{editDelegations}}, qw(Paint MouseDown Enter Leave Enable Disable KeyDown));
 	%profile = $self-> SUPER::init(%profile);
 	$self-> colors( $profile{colors});
 	$self-> value( $profile{value});
-	$self-> grayscale( $profile{grayscale});
 	return %profile;
 }
 
@@ -725,14 +709,12 @@ sub InputLine_Paint
 		($_[0],$_[1],$_[2],$_[1]-> size, $_[1]-> focused);
 	my $back = $self-> enabled ? $self-> backColor : $self-> disabledBackColor;
 	my $clr  = $combo-> value;
-	$clr = $combo->prelight_color($clr) if $self->{prelight};
 	$clr = $back if $clr == cl::Invalid;
 	$canvas-> rect3d( 0, 0, $w-1, $h-1, 1, $self-> light3DColor, $self-> dark3DColor);
 	$canvas-> color( $back);
 	$canvas-> rectangle( 1, 1, $w - 2, $h - 2);
 	$canvas-> rectangle( 2, 2, $w - 3, $h - 3);
 	$canvas-> color( $clr);
-	$canvas-> rop2(rop::CopyPut);
 	$canvas-> fillPattern([(0xEE, 0xBB) x 4]) unless $self-> enabled;
 	$canvas-> bar( 3, 3, $w - 4, $h - 4);
 	$canvas-> rect_focus(2, 2, $w - 3, $h - 3) if $focused;
@@ -788,24 +770,6 @@ sub InputLine_MouseWheel
 	$widget-> clear_event;
 }
 
-sub InputLine_MouseEnter
-{
-	my ($self, $widget) = @_;
-	if ( !$widget->capture && $self->enabled) {
-		$widget->{prelight} = 1;
-		$widget->repaint;
-	}
-}
-
-sub InputLine_MouseLeave
-{
-	my ($self, $widget) = @_;
-	if ( !$widget->capture && $self->enabled) {
-		delete $widget->{prelight};
-		$widget->repaint;
-	}
-}
-
 sub List_Create
 {
 	my ($combo,$self) = @_;
@@ -844,32 +808,23 @@ sub List_Paint
 	$canvas-> rect3d( 0, 0, $w-1, $h-1, 1, @c3d, cl::Back)
 		unless exists $self-> {inScroll};
 	my $i;
-	my $sc = $self->{scaling};
-	my $pc = 18 * $sc;
+	my $pc = 18 * $self->{scaling};
 	my $dy = $combo-> {btn}-> height;
 
 	my $maxc = $combo-> colors;
 	my $shft = $combo-> {scr}-> value;
 	for ( $i = 0; $i < 20; $i++) {
 		next if $i >= $maxc;
-		my $X = $i % 4;
-		my $Y = int($i / 4);
-		my ( $x, $y) = ($X * $pc + 3 * $sc, (4 - $Y) * $pc + 9 * $sc + $dy);
+		my ( $x, $y) = (($i % 4) * $pc + 3 * $self->{scaling}, ( 4 - int( $i / 4)) * $pc + 9 * $self->{scaling} + $dy);
 		my $clr = 0;
 		$combo-> notify('Colorify', $i + $shft, \$clr);
-
-		my @c = @c3d;
-		@c = reverse @c if
-			$self->{prelight} &&
-			$self->{prelight}->[0] == $X &&
-			$self->{prelight}->[1] == $Y;
-		$canvas-> rect3d( $x, $y, $x + $pc - 2 * $sc, $y + $pc - 2 * $sc, 1, @c, $clr);
+		$canvas-> rect3d( $x, $y, $x + $pc - 2 * $self->{scaling}, $y + $pc - 2 * $self->{scaling}, 1, @c3d, $clr);
 	}
 }
 
-sub list_pos2xy
+sub List_MouseDown
 {
-	my ( $combo, $self, $x, $y) = @_;
+	my ( $combo, $self, $btn, $mod, $x, $y) = @_;
 	$x -= 3 * $self->{scaling};
 	$y -= $combo-> {btn}-> height + 9 * $self->{scaling};
 	return if $x < 0 || $y < 0;
@@ -878,62 +833,24 @@ sub list_pos2xy
 	$y = int($y / $pc);
 	return if $x > 3 * $self->{scaling} || $y > 4 * $self->{scaling};
 	$y = 4 - $y;
+	$combo-> listVisible(0);
 	my $shft = $combo-> {scr}-> value;
 	my $maxc = $combo-> colors;
 	my $xcol = $shft + $x + $y * 4;
 	return if $xcol >= $maxc;
-
-	return $x, $y, $xcol;
-}
-
-sub List_MouseDown
-{
-	my ( $combo, $self, $btn, $mod, $x, $y) = @_;
-	my $xcol;
-	($x, $y, $xcol) = $combo->list_pos2xy($self, $x, $y);
-	return unless defined $x;
-	$combo-> listVisible(0);
 	my $xval = 0;
 	$combo-> notify('Colorify', $xcol, \$xval);
 	$combo-> value( $xval);
 }
 
-sub List_MouseMove
-{
-	my ( $combo, $self, $mod, $x, $y) = @_;
-	my $xcol;
-	($x, $y, $xcol) = $combo->list_pos2xy($self, $x, $y);
-	if ( defined $xcol ) {
-		return if
-			defined $self->{prelight} &&
-			$self->{prelight}->[0] == $x &&
-			$self->{prelight}->[1] == $y;
-		$self->{prelight} = [ $x, $y ];
-	} else {
-		return unless defined $self->{prelight};
-		delete $self->{prelight};
-	}
-	$self->repaint;
-}
-
-sub List_MouseLeave
-{
-	my ($self, $widget) = @_;
-	if ( !$widget->capture && $self->enabled) {
-		delete $widget->{prelight};
-		$widget->repaint;
-	}
-}
-
 sub MoreBtn_Click
 {
 	my ($combo,$self) = @_;
 	my $d;
 	$combo-> listVisible(0);
 	$d = Prima::Dialog::ColorDialog-> create(
-		text      => 'Mixed color palette',
-		value     => $combo-> value,
-		grayscale => $combo->grayscale,
+		text  => 'Mixed color palette',
+		value => $combo-> value,
 	);
 	$combo-> value( $d-> value) if $d-> execute != mb::Cancel;
 	$d-> destroy;
@@ -991,15 +908,7 @@ my @palColors = (
 sub on_colorify
 {
 	my ( $self, $index, $sref) = @_;
-	if ( $self->{grayscale}) {
-		my $g;
-		if ( $index < 20 ) {
-			$g = $index * 255 / 20;
-		} else {
-			$g = ($index - 20) * 255 / 128;
-		}
-		$$sref = cl::from_rgb($g,$g,$g);
-	} elsif ( $index < 20) {
+	if ( $index < 20) {
 		$$sref = $palColors[ $index];
 	} else {
 		my $i = $index - 20;
@@ -1022,13 +931,6 @@ sub on_colorify
 sub value        {($#_)?$_[0]-> set_value       ($_[1]):return $_[0]-> {value};  }
 sub colors       {($#_)?$_[0]-> set_colors      ($_[1]):return $_[0]-> {colors};  }
 
-sub grayscale
-{
-	return $_[0]->{grayscale} unless $#_;
-	my ( $self, $gs ) = @_;
-	$self->{grayscale} = $gs;
-	$self->value(cl::to_gray_rgb($self->value)) if $gs;
-}
 
 1;
 
@@ -1047,9 +949,9 @@ Prima::Dialog::ColorDialog - standard color selection facilities
 	);
 	printf "color: %06x", $p-> value if $p-> execute == mb::OK;
 
-=for podview <img src="colordlg.png">
+=for podview <img src="colordlg.png" cut=1>
 
-=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/Dialog/colordlg.png">
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/colordlg.png">
 
 =head1 DESCRIPTION
 
@@ -1064,10 +966,6 @@ invoke C<Prima::Dialog::ColorDialog> window.
 
 =over
 
-=item grayscale BOOLEAN
-
-If set, allows only gray colors for selection
-
 =item quality BOOLEAN
 
 Used to increase visual quality of the dialog if run on paletted displays.
@@ -1188,10 +1086,6 @@ result scalar, where the notification is expected to write the resulting color.
 
 Defines amount of colors in the fixed palette of the combo box.
 
-=item grayscale BOOLEAN
-
-If set, allows only gray colors for selection
-
 =item value COLOR
 
 Contains the color selection as 24-bit integer value.
