@@ -60,7 +60,7 @@ sub init
 	$self-> {fixedOnly}  = $profile{fixedOnly};
 	$self-> {sampleText} = $profile{sampleText};
 
-	my $gr = $self-> insert( GroupBox =>
+	my $gr = $self-> insert( CheckBoxGroup =>
 		origin => [ 10, 10],
 		size   => [ 150, 150],
 		name   => 'Style',
@@ -303,7 +303,7 @@ sub on_begindragfont
 {
 	my ( $self) = @_;
 	$self-> {old_text} = $self-> text;
-	$self-> Sample-> Example-> pointer( cr::DragMove);
+	$self-> Sample-> Example-> pointer( cr::Invalid);
 	$self-> text( "Apply font...");
 }
 
@@ -322,12 +322,15 @@ sub Example_Paint
 {
 	my ( $owner, $self, $canvas) = @_;
 	my @size = $canvas-> size;
-	$canvas-> clear;
+	$canvas-> color( $canvas-> get_nearest_color( $self-> backColor));
+	$canvas-> bar( 0, 0, @size);
+	$canvas-> color( cl::Black);
+	my $f = $self-> font;
 	my $line = $owner-> sampleText;
 	$canvas-> text_shape_out(
 		$line,
 		( $size[0] - $canvas-> get_text_width( $line)) / 2,
-		( $size[1] - $canvas->font-> height) / 2
+		( $size[1] - $f-> height) / 2
 	);
 }
 
@@ -453,19 +456,19 @@ Prima::Dialog::FontDialog - standard font dialog
 =head1 SYNOPSIS
 
 	use Prima qw(Application Dialog::FontDialog);
-	my $f = Prima::Dialog::FontDialog-> new;
+	my $f = Prima::Dialog::FontDialog-> create;
 	return unless $f-> execute == mb::OK;
 	$f = $f-> logFont;
 	print "$_:$f->{$_}\n" for sort keys %$f;
 
-=for podview <img src="fontdlg.gif">
+=for podview <img src="fontdlg.gif" cut=1>
 
-=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/Dialog/fontdlg.gif">
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/fontdlg.gif">
 
 =head1 DESCRIPTION
 
-The dialog provides standard font selection by name, style, size, and encoding.
-The selected font is returned by the L<logFont> property.
+The dialog provides selection of font by name, style, size, and encoding.
+The font selected is returned by L<logFont> property.
 
 =head1 API
 
@@ -487,13 +490,13 @@ FONT format is fully compatible with C<Prima::Drawable::font>.
 
 =item sampleText STRING
 
-Sample line of text drawn with the currently selected font
+Sample line of text featuring current font selection.
 
 Default value: AaBbYyZz
 
 =item showHelp BOOLEAN
 
-A create-only property.
+Create-only property.
 
 Specifies if the help button is displayed in the dialog.
 
@@ -508,17 +511,18 @@ Default value: 0
 =item BeginDragFont
 
 Called when the user starts dragging a font from the font sample widget by
-the left mouse button.
+left mouse button.
 
-The default action reflects the dragging status in the dialog title
+Default action reflects the status in the dialog title
 
 =item EndDragFont $WIDGET
 
-Called when the user releases the mouse button over a Prima widget.
-The default action applies the currently selected font to $WIDGET.
+Called when the user releases the mouse drag over a Prima widget.
+Default action applies currently selected font to $WIDGET.
 
 =back
 
+
 =head1 AUTHOR
 
 Dmitry Karasik, E<lt>dmitry@karasik.eu.orgE<gt>.
