@@ -25,7 +25,6 @@ sub filtered_codecs2all
 package Prima::Dialog::ImageOpenDialog;
 use vars qw( @ISA);
 @ISA = qw(Prima::Dialog::OpenDialog);
-use Prima::sys::FS;
 
 {
 my %RNT = (
@@ -63,6 +62,7 @@ sub init
 	my $pk = $self-> insert( ImageViewer =>
 		origin     => [ 524, 120],
 		name       => 'PickHole',
+		borderWidth=> 1,
 		alignment  => ta::Center,
 		valignment => ta::Center,
 		growMode   => gm::GrowLoX | gm::GrowLoY,
@@ -130,7 +130,7 @@ sub update_preview
 	return unless defined $x;
 
 	$x = $self-> directory . $x;
-	return unless _f $x;
+	return unless -f $x;
 
 	$x = Prima::Icon-> load( $x,
 		loadExtras => 1,
@@ -264,7 +264,6 @@ sub profile_default  {
 		filter   => [ Prima::Dialog::ImageDialog::filtered_codecs($codecs) ],
 		image    => undef,
 		filterDialog => 1,
-		useBase64 => 0,
 		noTestFileCreate => 1,
 	}
 }
@@ -274,7 +273,7 @@ sub init
 	my $self = shift;
 	my %profile = $self-> SUPER::init(@_);
 	$self-> {ConvertTo} = $self-> insert( ComboBox =>
-		origin => [ 524, 120],
+		origin => [ 524, 80],
 		items  => [],
 		enabled => 0,
 		name => 'ConvertTo',
@@ -283,7 +282,7 @@ sub init
 		growMode   => gm::GrowLoX,
 	);
 	$self-> insert( Label =>
-		origin => [ 524, 150],
+		origin => [ 524, 110],
 		text   => '~Convert to:',
 		size  => [ 96, 20],
 		name => 'ConvertToLabel',
@@ -298,14 +297,6 @@ sub init
 		delegations => [qw(Click)],
 		growMode   => gm::GrowLoX,
 	);
-	$self-> {UseBase64} = $self-> insert( CheckBox =>
-		origin => [ 524, 60],
-		text   => 'As ~base64',
-		size   => [ 96, 36],
-		name   => 'UseBase64',
-		delegations => [qw(Click)],
-		growMode   => gm::GrowLoX,
-	);
 
 	$self-> {codecFilters} = [];
 	$self-> {allCodecs} = Prima::Image-> codecs;
@@ -317,7 +308,6 @@ sub init
 
 	$self-> image( $profile{image});
 	$self-> filterDialog( $profile{filterDialog});
-	$self-> useBase64( $profile{useBase64});
 	$self-> ExtensionsLabel-> text("Sav~e as type ($codec->{fileShortType})");
 
 	return %profile;
@@ -352,7 +342,6 @@ sub update_conversion
 	my $j = 0;
 	for ( @st) {
 		return if $_ == $t;
-		next if $_ & (im::RealNumber|im::ComplexNumber|im::TrigComplexNumber); # not by default, at least
 		$max = $j if ( $st[$max] & im::BPP) < ( $_ & im::BPP);
 		$j++;
 	}
@@ -361,10 +350,6 @@ sub update_conversion
 		$x = '24-bit' if $x == 16777216;
 		$x .= ' gray' if $_ & im::GrayScale;
 		$x .= ' colors';
-		$x = 'float' if $_ == im::Float;
-		$x = 'double' if $_ == im::Double;
-		$x = 'complex' if $_ == im::Complex;
-		$x = 'double complex' if $_ == im::DComplex;
 		$_ = $x;
 	}
 
@@ -395,12 +380,6 @@ sub filterDialog
 	$_[0]-> {UseFilter}-> checked( $_[1]);
 }
 
-sub useBase64
-{
-	return $_[0]-> {UseBase64}-> checked unless $#_;
-	$_[0]-> {UseBase64}-> checked( $_[1]);
-}
-
 sub image
 {
 	return $_[0]-> {image} unless $#_;
@@ -494,27 +473,10 @@ sub save
 		$j++;
 	}
 
-	if ( $self->useBase64) {
-		my $error;
-		my $ok = $dup-> save_stream;
-		if (defined $ok) {
-			my $f;
-			my $r_ok = open $f, ">", $self->fileName;
-			$error = "$!", goto FAIL unless $r_ok;
-			$error = "$!", goto FAIL unless print $f $ok;
-			$error = "$!", goto FAIL unless close $f;
-			$ret = 1;
-		} else {
-			$error = "$@";
-		FAIL:
-			Prima::MsgBox::message("Error saving " . $self-> fileName . ":$error");
-		}
+	if ( $dup-> save( $self-> fileName, %profile)) {
+		$ret = 1;
 	} else {
-		if ( $dup-> save( $self-> fileName, %profile)) {
-			$ret = 1;
-		} else {
-			Prima::MsgBox::message("Error saving " . $self-> fileName . ":$@");
-		}
+		Prima::MsgBox::message("Error saving " . $self-> fileName . ":$@");
 	}
 
 EXIT:
@@ -527,21 +489,22 @@ EXIT:
 
 =head1 NAME
 
-Prima::Dialog::ImageDialog - image file open and save dialogs
+Prima::Dialog::ImageDialog - file open and save dialogs.
 
 =head1 DESCRIPTION
 
-The module provides standard dialogs specially adjusted for image
+The module provides dialogs specially adjusted for image
 loading and saving.
 
 =head1 Prima::Dialog::ImageOpenDialog
 
-Provides a preview widget, allowing the user to view the image file before
+Provides a preview feature, allowing the user to view the image file before
 loading, and the selection of a frame index for the multi-framed image files.
-Instead of the C<execute> call, the L<load> method is used to invoke the dialog
-and returns the loaded image as a C<Prima::Image> object.  The loaded object
-contains the C<{extras}> hash with the file and image information filled by the
-loader. See L<Prima::image-load> for more information.
+Instead of C<execute> call, the L<load> method is used to invoke the dialog and
+returns the loaded image as a C<Prima::Image> object.  The loaded object by
+default contains C<{extras}> hash variable set, which contains extra
+information returned by the loader. See L<Prima::image-load> for more
+information.
 
 =head2 SYNOPSIS
 
@@ -551,11 +514,11 @@ loader. See L<Prima::image-load> for more information.
 	return unless $img;
 	print "$_:$img->{extras}->{$_}\n" for sort keys %{$img-> {extras}};
 
-=for podview <img src="imagedlg.gif">
+=for podview <img src="imagedlg.gif" cut=1>
 
-=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/Dialog/imagedlg.gif">
+=for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/imagedlg.gif">
 
-=head2 Properties
+=head2 Proprties
 
 =over
 
@@ -576,19 +539,19 @@ Default value: 1
 
 Executes the dialog, and, if successful, loads the image file and frame
 selected by the user. Returns the loaded image as a C<Prima::Image> object.
-PROFILE is a hash, passed to the C<Prima::Image::load> method. In particular,
-it can be used to disable the default loading of extra information in the
-C<{extras}> hash variable or to specify a non-default loading option.  For
-example, C<< {extras}->{className} = 'Prima::Icon' >> would return the
-loaded image as an icon object. See L<Prima::image-load> for more.
+PROFILE is a hash, passed to C<Prima::Image::load> method. In particular, it
+can be used to disable the default loading of extra information in C<{extras}>
+variable, or to specify a non-default loading option.  For example,
+C<{extras}-E<gt>{className} = 'Prima::Icon'> would return the loaded image as
+an icon object. See L<Prima::image-load> for more.
 
-C<load> can report the progress of the image loading to the caller, and/or to an
+C<load> can report progressive image loading to the caller, and/or to an
 instance of C<Prima::ImageViewer>, if desired. If either (or both)
 C<onHeaderReady> and C<onDataReady> notifications are specified, these are
 called from the respective event handlers of the image being loaded ( see
 L<Prima::image-load/"Loading with progress indicator"> for details).  If
-the profile key C<progressViewer> is supplied, its value is treated as a
-C<Prima::ImageViewer> instance, and it is used to display the loading
+profile key C<progressViewer> is supplied, its value is treated as a
+C<Prima::ImageViewer> instance, and it is used to display image loading
 progress. See L<Prima::ImageViewer/watch_load_progress>.
 
 =back
@@ -609,20 +572,20 @@ See L<Prima::Image/DataReady>.
 
 =head1 Prima::Dialog::ImageSaveDialog
 
-Provides the standard image save dialog where the user can select the desired
-image format, the bit depth, and other format-specific options. The
-format-specific options can be set if a dialog for the file format is provided
-by the toolkit.  The standard toolkit dialogs reside under the C<Prima::Image>
-namespace, in the F<Prima/Image> subdirectory. For example,
-C<Prima::Image::gif> provides the selection of a transparent color, and
-C<Prima::Image::jpeg> provides the image quality control. If the image passed
-to the L<image> property contains the C<{extras}> hash variable, its data are used
-as the default values.  In particular, the C<< {extras}->{codecID} >> field,
-responsible for the file format, affects the default file format selection.
+Provides a save dialog where the user can select image format,
+the bit depth and other format-specific options. The format-specific
+options can be set if a dialog for the file format is provided.
+The standard toolkit dialogs reside under in C<Prima::Image> namespace,
+in F<Prima/Image> subdirectory. For example, C<Prima::Image::gif> provides
+the selection of transparency color, and C<Prima::Image::jpeg> the image
+quality control. If the image passed to the L<image> property contains
+C<{extras}> variable, the data are read and used as the default values.
+In particular, C<{extras}-E<gt>-{codecID}> field, responsible for the
+file format, if present, affects the default file format selection.
 
 =head2 SYNOPSIS
 
-	my $dlg = Prima::Dialog::ImageSaveDialog-> new;
+	my $dlg = Prima::Dialog::ImageSaveDialog-> create;
 	return unless $dlg-> save( $image );
 	print "saved as ", $dlg-> fileName, "\n";
 
@@ -632,9 +595,9 @@ responsible for the file format, affects the default file format selection.
 
 =item image IMAGE
 
-Selects the image to be saved. The property is to be used
-for the standard invocation of dialog, via the C<execute> method. It is not
-needed when the execution and saving are invoked via the L<save> method.
+Selects the image to be saved. This property is to be used
+for the standard invocation of dialog, via C<execute>. It is not
+needed when the execution and saving is invoked via L<save> method.
 
 =back
 
@@ -644,13 +607,13 @@ needed when the execution and saving are invoked via the L<save> method.
 
 =item save IMAGE, %PROFILE
 
-Invokes the dialog, and, if the execution is successful, saves the IMAGE
-according to the user selection and PROFILE hash.  PROFILE is not used as a
-source of the default options, but is passed directly to the
-C<Prima::Image::save> call, possibly overriding the selection of the user.
-
-Returns 1 on success, 0 on error. If the error occurs, the
-user is notified before the method returns.
+Invokes the dialog, and, if the execution was successful, saves
+the IMAGE according to the user selection and PROFILE hash.
+PROFILE is not used for the default options, but is passed
+directly to C<Prima::Image::save> call, possibly overriding
+selection of the user.
+Returns 1 in case of success, 0 in case of error.
+If the error occurs, the user is notified before the method returns.
 
 =back
 
