@@ -2,12 +2,12 @@ package Prima::Drawable::Markup;
 
 use strict;
 use warnings;
-use Prima qw(Bidi Drawable::TextBlock);
+use Prima qw(Drawable::TextBlock);
 use base qw(Prima::Drawable::TextBlock);
 
 =head1 NAME
 
-Prima::Markup - Allow markup in Prima Widgets
+Prima::Drawable::Markup - allow markup in widgets
 
 =head1 SYNOPSIS
 
@@ -18,13 +18,13 @@ Prima::Markup - Allow markup in Prima Widgets
     );
     run Prima;
 
-=for podview <img src="markup.gif" cut=1>
+=for podview <img src="Prima/markup.gif" cut=1>
 
 =for html <p><img src="https://raw.githubusercontent.com/dk/Prima/master/pod/Prima/markup.gif">
 
 =head1 DESCRIPTION
 
-C<Prima::Markup> adds the ability to recognize POD-like markup to Prima
+C<Prima::Drawable::Markup> adds the ability to recognize POD-like markup to Prima
 widgets. Supported markup sequences are C<B> (bold text), C<I> (italic text),
 C<U> (underlined text), C<F> (change font), C<S> (change font size), C<C>
 (change foreground color), C<Q> (change background color), C<M> (move pointer),
@@ -60,7 +60,7 @@ The text inside C<W> sequence will not be wrapped during C<text_wrap> calls.
 The C<P> sequence is used as follows:C<< PE<lt>nE<gt> >>, where C<n> is a
 0-based index into the C<picturePalette>.
 
-The methods C<text_out> and C<get_text_width> are affected by C<Prima::Markup>.
+The methods C<text_out> and C<get_text_width> are affected by C<Prima::Drawable::Markup>.
 C<text_out> will write formatted text to the canvas, and C<get_text_width> will
 return the width of the formatted text.  B<NOTE>: These methods do not save state
 between calls, so your markup cannot span lines (since each line is drawn or
@@ -74,7 +74,6 @@ sub new
 	%opt = ( %opt,
 		fontmap       => [{}],
 		colormap      => [0,0],
-		bidi          => $Prima::Bidi::enabled,
 	);
 	my $self = $class->SUPER::new(%opt);
 	$self-> $_( $opt{$_} || [] ) for qw(fontPalette colorPalette picturePalette);
@@ -82,13 +81,6 @@ sub new
 	return $self;
 }
 
-sub bidi {
-	return $_[0]->{bidi} unless $#_;
-	my ( $self, $bidi ) = @_;
-	$bidi = 0 unless $Prima::Bidi::enabled;
-	$self->{bidi} = $bidi;
-}
-
 sub parse_color
 {
 	my ( $self, $mode, $command, $stacks, $state, $block, $c ) = @_;
@@ -376,13 +368,6 @@ sub markup
 	my ( $self, $markup ) = @_;
 	my ( $text, $block ) = $self-> parse( $markup );
 
-	if ( $self->{bidi} && Prima::Bidi::is_bidi($text) ) {
-		$self-> {nonbidiblock} = $block;
-		$block = tb::bidi_visualize( $block, text => $text );
-	} else {
-		$self-> {nonbidiblock} = undef;
-	}
-
 	$self-> {markup} = $markup;
 	$self-> text( $text );
 	$self-> {block} = $block;
@@ -444,8 +429,6 @@ sub text_wrap
 {
 	my ( $self, $canvas, $width, $opt, $indent) = @_;
 
-	local $self->{block} = $self->{nonbidiblock} if $self->{nonbidiblock};
-
 	my @ret = @{ $self-> SUPER::text_wrap( $canvas, $width, $opt, $indent ) };
 
 	my ( @blocks, @other);
@@ -461,32 +444,6 @@ sub text_wrap
 	}
 	return @other unless @blocks;
 
-	# Feed less text to bidi_visualize because it can take things into account
-	# that were wrapped away for a sub-block. F.ex:
-	# wrap("bidi . nonbidi") results in ("bidi n", "onbidi"). While the 1st
-	# scalar is okay when bidified, noone knows whether the 2nd will look
-	# the same when in the big string context and when torn away after the wrap.
-	if ( $self->{bidi} && Prima::Bidi::is_bidi($self->{text})) {
-		my $text_offset = $blocks[-1]->{block}->[tb::BLK_TEXT_OFFSET];
-		my $last_offset = 0;
-		tb::walk( $blocks[-1]->{block}, text => sub {
-			my ( $ofs, $len ) = @_;
-			$last_offset = $ofs + $len;
-		});
-		my @text_offsets = (
-			( map { $_->{block}->[ tb::BLK_TEXT_OFFSET ] } @blocks ),
-			$text_offset + $last_offset);
-
-		my $t = $self->{text};
-		for ( my $j = 0; $j < @blocks; $j++) {
-			my $substr  = substr( $t, $text_offsets[$j], $text_offsets[$j+1] - $text_offsets[$j]);
-			next unless Prima::Bidi::is_bidi($substr);
-			$blocks[$j]->{block} = tb::bidi_visualize(
-				$blocks[$j]->{block}, text => $substr,
-			);
-		}
-	}
-
 	# initials will be overwritten by acquire(), force copy them
 	for my $block ( @blocks ) {
 		my $b = $block->{block};
@@ -523,11 +480,6 @@ Each element of the array should be a hashref suitable for setting a font.
 Gets or sets the picture palette to be used for C<P> sequences within this widget.
 Each element of the array should be a C<Prima::Image> descendant.
 
-=item bidi BOOLEAN = 1
-
-If 0, bidirection text processing if off. If 1, set to 1 iff C<Prima::Bidi> is
-loaded and enabled.
-
 =back
 
 =head1 SEE ALSO
