@@ -41,26 +41,6 @@ use constant italic_angle    => "\x{c}\x{2}";
 use constant paint_type      => "\x{c}\x{5}";
 use constant font_matrix     => "\x{c}\x{7}";
 
-sub int32($)
-{
-	my ( $self, $n ) = @_;
-	$n = Prima::Utils::nearest_i( $n );
-	if (-107 <= $n && $n <= 107) {
-		return chr($n + 139);
-	} elsif (108 <= $n && $n <= 1131) {
-		$n -= 108;
-		return chr(($n >> 8) + 247).chr($n & 0xff);
-	} elsif (-1131 <= $n && $n <= -108) {
-		$n = -$n - 108;
-		return chr(($n >> 8) + 251).chr($n & 0xff);
-	} elsif (-32768 <= $n && $n < 32767) {
-		return pack('Cn', 28, $n);
-	} else {
-		return pack('CN', 29, $n);
-	}
-}
-
-
 sub mk_index
 {
 	my $ret = '';
@@ -77,13 +57,11 @@ sub mk_index
 sub mk_offset
 {
 	my $n = shift;
-	return
-		chr(29).
-		chr(($n >> 24) & 0xff).
-		chr(($n >> 16) & 0xff).
-		chr(($n >>  8) & 0xff).
-		chr($n & 0xff)
-		;
+	if (-32768 <= $n && $n < 32767) {
+		return chr(28).chr(($n >> 8) & 0xff).chr($n & 0xff);
+	} else {
+		return chr(29).chr(($n >> 24) & 0xff).chr(($n >> 16) & 0xff).chr(($n >> 8) & 0xff).chr($n & 0xff);
+	}
 }
 
 
@@ -142,32 +120,46 @@ sub evacuate_next_subfont
 	}
 
 	# charsets and privates
-	my @bbox        = Prima::Utils::nearest_i( map { $_ // 0 } @{ $v->{bbox} } );
+	my @bbox        = map { Prima::Utils::floor(($_ // 0) + .5) } @{ $v->{bbox} };
 	my $charset_str = pack('Cn*', 0, @charset); # mode 0, glyph list
-	my $private_str = $self->num( $bbox[1], -$bbox[1] ) . blue_values;
+	my $private_str = num( $bbox[1], -$bbox[1] ) . blue_values;
 
 	# header
 	my $header          = pack("C*", 1, 0, 4, 2) . mk_index($fn);
 	push @strings,      $fn;
-	push @const_data,   $self->int32(391 + $#strings) . full_name;
-	push @const_data,   $self->int32(391 + $#strings) . family_name;
-	push @const_data,   $self->int32($v->{bold} ? 384 : 388) . weight;
-	push @const_data,   $self->int32($v->{fixed})     . is_fixed_pitch;
-	push @const_data,   $self->int32($v->{italic})    . italic_angle;
-	push @const_data,   $self->num( @bbox )           . font_bbox;
+	push @const_data,   int32(391 + $#strings) . full_name;
+	push @const_data,   int32(391 + $#strings) . family_name;
+	push @const_data,   int32($v->{bold} ? 384 : 388) . weight;
+	push @const_data,   int32($v->{fixed})     . is_fixed_pitch;
+	push @const_data,   int32($v->{italic})    . italic_angle;
+	push @const_data,   num( @bbox )           . font_bbox;
 
 	my $strings_str = mk_index(@strings);
 	my $subr_str    = pack('n', 0);
 
-	$offsets{charstrings} = 0;
-	$offsets{charset}     = 0;
-	$offsets{private}     = 0;
+	# the offsets are affected by the encoded header length, but
+	# the header itself contains references to the offsets, that
+	# in turn may change the header length. So make several shoots at it
+	$offsets{charstrings} = 100;
+	$offsets{charset}     = 500;
+	$offsets{private}     = 2500;
+
 	my $dict_str = mk_header(\@const_data, \%offsets, length($private_str));
+	my $safeguard = 10;
+	while ( $safeguard-- ) {
+		$offsets{charset}     = length($header) + length($dict_str) + length($strings_str) + length($subr_str);
+		$offsets{charstrings} = $offsets{charset}     + length($charset_str);
+		$offsets{private}     = $offsets{charstrings} + $charstrings_len;
+		my $real_dict_str = mk_header(\@const_data, \%offsets, length($private_str));
+		my $length_match  = length($real_dict_str) == length($dict_str);
+		$dict_str = $real_dict_str;
+		last if $length_match;
+	}
+	if ( $safeguard <= 0 ) {
+		warn "panic: cannot encode font $fn as CFF";
+		return;
+	}
 
-	$offsets{charset}     = length($header) + length($dict_str) + length($strings_str) + length($subr_str);
-	$offsets{charstrings} = $offsets{charset}     + length($charset_str);
-	$offsets{private}     = $offsets{charstrings} + $charstrings_len;
-	$dict_str = mk_header(\@const_data, \%offsets, length($private_str));
 
 	$ret_content = join('', $header, $dict_str, $strings_str, $subr_str, $charset_str);
 	$ret_content .= mk_index(endchar, map {$_->[3]} @glyphs);
