@@ -1,3 +1,6 @@
+# XXX todo
+# regions
+# paths
 package Prima::PS::PDF;
 
 use strict;
@@ -5,7 +8,6 @@ use warnings;
 use Encode;
 use Prima;
 use Prima::PS::CFF;
-use Prima::PS::Format;
 use Prima::PS::TempFile;
 use base qw(Prima::PS::Drawable);
 
@@ -72,47 +74,36 @@ sub change_transform
 	my ( $self, $gsave ) = @_;
 	return if $self-> {delay};
 
-	my ($ps, $pm) = @{ $self }{ qw(pageSize pageMargins) };
-	my @pm = (
-		@$pm[0,1],
-		$ps->[0] - $pm->[2] - $pm->[0],
-		$ps->[1] - $pm->[3] - $pm->[1]
-	);
-	@pm[0,1] = (0,0) if $self->{lock_margins};
+	my @tp = $self-> translate;
+	my @cr = $self-> clipRect;
+	my @sc = $self-> scale;
+	my $ro = $self-> rotate;
 
-	my ($doClip, @cr);
-	my $rg = $self-> region;
+	$cr[2] -= $cr[0];
+	$cr[3] -= $cr[1];
+	my $doClip = grep { $_ != 0 } @cr;
+	my $doTR   = grep { $_ != 0 } @tp;
+	my $doSC   = grep { $_ != 0 } @sc;
 
-	unless ( $rg ) {
-		@cr = $self-> clipRect;
-		$cr[2] -= $cr[0];
-		$cr[3] -= $cr[1];
-		$doClip = grep { $_ != 0 } @cr;
-
-		@cr = $self-> pixel2point( @cr);
-		float_inplace(@cr);
+	if ( !$doClip && !$doTR && !$doSC && !$ro) {
+		$self-> emit_content('q') if $gsave;
+		return;
 	}
 
+	@cr = $self-> pixel2point( @cr);
+	@tp = $self-> pixel2point( @tp);
+	my $mcr3 = -$cr[3];
+
 	$self-> emit_content('Q') unless $gsave;
 	$self-> emit_content('q');
-
-	$self-> emit_content("@pm re W n");
-
-	my $m = $self-> get_matrix;
-	if ( $self->{reversed}) {
-		$$m[5] -= ($self->point2pixel($pm[2]))[0];
-		$m->rotate(90);
-	}
-	my @tm = $self-> pixel2point( @$m[4,5] );
-	$tm[$_] += $pm[$_] for 0,1;
-	my @xm = @$m[0..3];
-	float_inplace(@pm, @xm, @tm);
-	$self-> emit_content("@xm @tm cm");
-
-	if ( $rg ) {
-		$self-> emit_content($rg-> apply_clip($self));
-	} elsif ( $doClip ) {
-		$self-> emit_content("h @cr re W n");
+	$self-> emit_content("@cr W") if $doClip;
+	$self-> emit_content("1 0 0 1 @tp cm") if $doTR;
+	$self-> emit_content("$sc[0] 0 0 $sc[1] 0 0 cm") if $doSC;
+	if ($ro != 0) {
+		my $sin1 = sin($ro);
+		my $cos  = cos($ro);
+		my $sin2 = -$sin1;
+		$self-> emit_content("$cos $sin1 $sin2 $cos 0 0 cm");
 	}
 	$self-> {changed}-> {$_} = 1 for qw(fill linePattern lineWidth lineJoin lineEnd miterLimit font);
 }
@@ -125,12 +116,7 @@ sub fill
 		$r1 == rop::NoOper &&
 		$r2 == rop::NoOper;
 
-	if ( $self-> {changed}-> {alpha}) {
-		my $al = $self-> alpha;
-		$self-> emit_content( "/GSA$al gs");
-		$self-> {changed}-> {alpha} = 0;
-	}
-	if ( $r2 != rop::NoOper && $self-> {fpType} ne 'F' && !$self->{color_patterns}->{$self->{fpType}}) {
+	if ( $r2 != rop::NoOper && $self-> {fpType} ne 'F') {
 		my $bk =
 			( $r2 == rop::Blackness) ? 0 :
 			( $r2 == rop::Whiteness) ? 0xffffff : $self-> backColor;
@@ -146,8 +132,6 @@ sub fill
 		if ($self-> {changed}-> {fill}) {
 			if ( $self-> {fpType} eq 'F') {
 				$self-> emit_content( lc $self-> cmd_rgb( $c));
-			} elsif ( $self->{color_patterns}->{$self->{fpType}} ) {
-				$self-> emit_content("/Pattern cs /P$self->{fpType} scn");
 			} else {
 				my ( $r, $g, $b) = (
 					int((($c & 0xff0000) >> 16) * 100 / 256 + 0.5) / 100,
@@ -178,12 +162,10 @@ sub stroke
 		$r1 == rop::NoOper &&
 		$r2 == rop::NoOper;
 
-	my $lw_changed;
 	if ( $self-> {changed}-> {lineWidth}) {
 		my ($lw) = $self-> pixel2point($self-> lineWidth);
-		$self-> emit_content( float_format($lw) . ' w');
+		$self-> emit_content( $lw . ' w');
 		$self-> {changed}-> {lineWidth} = 0;
-		$lw_changed = 1;
 	}
 
 	if ( $self-> {changed}-> {lineEnd}) {
@@ -202,16 +184,10 @@ sub stroke
 
 	if ( $self-> {changed}-> {miterLimit}) {
 		my $ml = $self-> miterLimit;
-		$self-> emit_content( float_format($ml) . " M");
+		$self-> emit_content( "$ml M");
 		$self-> {changed}-> {miterLimit} = 0;
 	}
 
-	if ( $self-> {changed}-> {alpha}) {
-		my $al = $self-> alpha;
-		$self-> emit_content( "/GSA$al gs");
-		$self-> {changed}-> {alpha} = 0;
-	}
-
 	if ( $r2 != rop::NoOper && $lp ne lp::Solid ) {
 		my $bk =
 			( $r2 == rop::Blackness) ? 0 :
@@ -229,20 +205,14 @@ sub stroke
 			( $r1 == rop::Blackness) ? 0 :
 			( $r1 == rop::Whiteness) ? 0xffffff : $self-> color;
 
-		if ( $self-> {changed}-> {linePattern} || $lw_changed) {
+		if ( $self-> {changed}-> {linePattern}) {
 			if ( length( $lp) == 1) {
-				$self-> emit_content('[] 0 d') if $self->{changed}->{linePattern};
+				$self-> emit_content('[] 0 d');
 			} else {
-				my $lw = $self->lineWidth || 2.0; # because to be divided by 2
-				my @x = map { ord } split('', $lp);
+				my @x = split('', $lp);
 				push( @x, 0) if scalar(@x) % 1;
-				my @lp;
-				for ( my $i = 0; $i < @x; $i += 2 ) {
-					push @lp, ($x[$i] - 1) * $lw / 2;
-					push @lp, ($x[$i+1]) * $lw / 2;
-				}
-				@lp = map { $_ || 1 } @lp;
-				$self-> emit_content("[@lp] 0 d");
+				@x = map { ord($_) } @x;
+				$self-> emit_content("[@x] 0 d");
 			}
 			$self-> {changed}-> {linePattern} = 0;
 		}
@@ -354,7 +324,7 @@ sub emit_new_dummy_object
 sub begin_doc
 {
 	my ( $self, $docName) = @_;
-	$@ = 'Bad object', return 0 if $self-> get_paint_state;
+	return 0 if $self-> get_paint_state;
 
 	$self-> {ps_data}  = '';
 	$self-> {can_draw} = 1;
@@ -364,9 +334,23 @@ sub begin_doc
 		unless defined $docName;
 	$docName = Encode::encode('UTF-16', $docName)
 		if Encode::is_utf8($docName);
+	my $data = scalar localtime;
+	my @b2 = (
+		int($self-> {pageSize}-> [0] - $self-> {pageMargins}-> [2] + .5),
+		int($self-> {pageSize}-> [1] - $self-> {pageMargins}-> [3] + .5)
+	);
+
 	$self-> {fp_hash}  = {};
 	$self-> {xref} = [];
 
+	my ($x,$y) = (
+		$self-> {pageSize}-> [0] - $self-> {pageMargins}-> [0] - $self-> {pageMargins}-> [2],
+		$self-> {pageSize}-> [1] - $self-> {pageMargins}-> [1] - $self-> {pageMargins}-> [3]
+	);
+
+	my $extras = '';
+	my $setup = '';
+
 	my ($sec,$min,$hour,$mday,$mon,$year) = localtime;
 	my $date = sprintf("%04d%02d%02d%02d%02d%02d", $year + 1900, $mon, $mday, $hour, $min, $sec);
 	my $four = pack('C*', 0xde, 0xad, 0xbe, 0xef);
@@ -396,23 +380,20 @@ ROOT
 	$self-> {pages}         = [$self->{page_object} ];
 	$self-> {page_refs}     = [];
 	$self-> {page_patterns} = {};
-	$self-> {color_patterns}= {};
 	$self-> {page_images}   = [];
 	$self-> {page_fonts}    = {};
 	$self-> {page_rops}     = {};
-	$self-> {page_alphas}   = {};
-	$self-> {all_rops}      = {};
+	$self-> {all_rops}     = {};
 	$self-> {all_fonts}     = {};
-	$self-> {all_alphas}    = {};
 	unless ($self-> {page_content} = $self->new_file_obj) {
 		$self-> abort_doc;
-		$@ = 'Cannot create temp file', return 0;
+		return 0;
 	}
 
 	$self-> {changed} = { map { $_ => 0 } qw(
 		fill lineEnd linePattern lineWidth lineJoin miterLimit font)};
 
-	$self-> {paint_state} = ps::Enabled;
+	$self-> SUPER::begin_paint;
 	$self-> save_state;
 
 	$self-> {delay} = 1;
@@ -431,12 +412,11 @@ sub end_page
 
 	$self-> emit_content('Q');
 
-	my @ps = @{ $self->{pageSize} };
 	$self-> emit_new_object($self->{page_object}, <<PAGE);
 <<
 /Type /Page
 /Parent 3 0 R
-/MediaBox [ 0 0 @ps ]
+/MediaBox [ 0 0 @{$self->{pageSize}} ]
 /StructParents 0
 /Contents $self->{page_content} 0 R
 /ProcSet [ /PDF /Text /ImageB /ImageC /ImageI ]
@@ -466,20 +446,16 @@ PAGE
 		}
 		$self-> emit(">>");
 	}
+	$self-> emit(">>"); # % Resources
 
-	if ( keys %{ $self->{page_rops} } || keys %{ $self->{page_alphas} } ) {
+	if ( keys %{ $self->{page_rops} } ) {
 		$self-> emit("/ExtGState <<");
 		while ( my ( $name, $xid ) = each %{ $self->{page_rops} } ) {
 			$self-> emit("/GS$name $xid 0 R");
 		}
-		while ( my ( $name, $xid ) = each %{ $self->{page_alphas} } ) {
-			$self-> emit("/GSA$name $xid 0 R");
-		}
 		$self-> emit(">>");
 	}
 
-	$self-> emit(">>"); # % Resources
-
 	if ( @{ $self->{page_refs} } ) {
 		$self-> emit("/XObject <<");
 		for my $xid ( @{ $self->{page_refs} } ) {
@@ -498,8 +474,8 @@ sub abort_doc
 	my $self = $_[0];
 	return unless $self-> {can_draw};
 	$self-> {can_draw} = 0;
-	delete $self->{paint_state};
-	$self-> abandon_state;
+	$self-> SUPER::end_paint;
+	$self-> restore_state;
 	delete $self-> {$_} for
 		qw (save_state ps_data changed );
 }
@@ -510,7 +486,7 @@ sub end_paint   {        $_[0]-> abort_doc; }
 sub end_doc
 {
 	my $self = $_[0];
-	$@ = 'Bad object', return 0 unless $self-> {can_draw};
+	return 0 unless $self-> {can_draw};
 	$self-> end_page;
 
 	my $pages = scalar @{ $self->{pages} };
@@ -553,7 +529,7 @@ END
 
 			my $font_desc = $self-> new_dummy_obj;
 			my $charset_str = join('', map { "/$_" } @$charset);
-			my @bbox = Prima::Utils::nearest_i( map { $_ // 0 } @{ $frec->{bbox} } );
+			my @bbox = map { Prima::Utils::floor(($_ // 0) + .5) } @{ $frec->{bbox} };
 
 			$self-> emit_new_object($font_desc, <<FONT);
 <<
@@ -658,7 +634,7 @@ TRAILER
 	$self->{ps_data} = '';
 
 	$self-> {can_draw} = 0;
-	delete $self->{paint_state};
+	$self-> SUPER::end_paint;
 	$self-> restore_state;
 	delete $self-> {$_} for
 		qw (save_state changed ps_data glyph_keeper glyph_font);
@@ -675,7 +651,6 @@ sub new_page
 	push @{$self-> {pages}}, $self->{page_object};
 	$self-> {page_refs}      = [];
 	$self-> {page_patterns}  = {};
-	$self-> {color_patterns} = {};
 	$self-> {page_images}    = [];
 	$self-> {page_fonts}     = {};
 	$self-> {page_rops}      = {};
@@ -695,103 +670,43 @@ sub new_page
 
 sub pages { scalar @{ $_[0]-> {pages} } }
 
-sub alpha
+sub fillPattern
 {
-	return $_[0]->{alpha} unless $#_;
-	my ( $self, $alpha ) = @_;
-	$alpha = int( $alpha + .5);
-	$alpha = 0 if $alpha < 0;
-	$alpha = 255 if $alpha > 255;
-	return if ($self->{alpha} // -1 ) == $alpha;
-	$self->{alpha} = $alpha;
-	return unless $self-> get_paint_state;
-
-	my $ca = int( $alpha / 2.55 ) / 100;
-	$self-> {all_alphas}->{ $alpha } //= {
-		xid => $self-> emit_new_dummy_object("/Type /ExtGState /ca $ca /CA $ca"),
-		id  => "GSA$alpha",
-	};
-	$self-> {page_alphas}-> {$alpha} = $self->{all_alphas}->{$alpha}->{xid};
-	$self-> {changed}->{alpha} = 1;
-}
-
-sub emit_pattern
-{
-	my ( $self, $fpid, $fp) = @_;
+	return $_[0]-> SUPER::fillPattern unless $#_;
+	$_[0]-> SUPER::fillPattern( $_[1]);
+	return unless $_[0]-> {can_draw};
 
+	my $self = $_[0];
+	my @fp  = @{$self-> SUPER::fillPattern};
+	my $solidBack = ! grep { $_ != 0 } @fp;
+	my $solidFore = ! grep { $_ != 0xff } @fp;
+	my $fpid;
+	my @scaleto = $self-> pixel2point( 8, 8);
 	my $xid;
-	unless ( exists $self-> {fp_hash}-> {$fpid}) {
-		$xid  = $self-> new_dummy_obj;
-		$self-> {fp_hash}-> {$fpid} = $xid;
-		$self->{page_patterns}->{$xid}++;
-	} else {
-		$xid = $self-> {fp_hash}-> {$fpid};
-		$self->{page_patterns}->{$xid}++;
-		return $xid;
-	}
-
-	my (@sz, $imgdef, $depth, $paint, $proc, $cs, $imagemask, $maskdef);
-	$imgdef  = '';
-	$maskdef = '';
-	$paint     = 2;
-	$depth     = '1';
-	$proc      = 'ImageB';
-	$cs        = 'G';
-	$imagemask = 'true';
-	if ( ref($fp) eq 'ARRAY') {
-		@sz        = (8,8);
-		$imgdef    = pack('C*', @$fp);
-	} else {
-		@sz = $fp->size;
-		my ( $ls, $stride, $data );
-		if ( $fp->type == im::BW ) {
-			$stride    = int($fp-> width / 8) +!!+ ($fp->width % 8);
-		} else {
-			$paint     = 1;
-			$fp        = $self->prepare_image($fp);
-			$depth     = $fp->get_bpp;
-			$imagemask = 'false';
-			if ( $depth != 8 ) {
-				$depth   = 8;
-				$proc    = 'ImageC';
-				$cs      = 'RGB';
-			} else {
-				$proc    = 'ImageI';
-				$cs      = 'G';
-			}
-			$stride  = $fp->get_bpp * $fp->width / 8;
-			$self->{color_patterns}->{$xid} = 1;
-		}
-		$ls      = $fp->lineSize;
-		$data    = $fp->data;
-		for my $y ( 0 .. $fp->height - 1 ) {
-			$imgdef .= substr($data, $y * $ls, $stride);
-		}
-	}
-	my $patdef  = <<PAT;
+	if ( !$solidBack && !$solidFore) {
+		$fpid = join( '', map { sprintf("%02x", $_)} @fp);
+		unless ( exists $self-> {fp_hash}-> {$fpid}) {
+			$xid  = $self-> new_dummy_obj;
+			my $bits = pack('C*', @fp);
+			my $patdef = <<PAT;
 q
 BI
-/IM $imagemask
-/W $sz[0]
-/H $sz[1]
-/BPC $depth
-/CS /$cs
-ID $imgdef
+/IM true
+/W 8
+/H 8
+/BPC 1
+ID $bits
 EI Q
 PAT
-
-	# PatternType 1 = Tiling pattern
-	# PaintType   1 = Colored
-	# PaintType   2 = Uncolored
-	$self-> emit_new_object( $xid, <<PATTERNDEF);
+			$self-> emit_new_object( $xid, <<PATTERNDEF);
 <<
 /Type /Pattern
 /BBox [0 0 1 1]
 /Length ${ \length $patdef }
-/PaintType $paint
-/PatternType 1
+/PaintType 2 % Uncolored
+/PatternType 1 % Tiling pattern
 /Resources <<
-/ProcSet [ /PDF /$proc ]
+/ProcSet [ /PDF /ImageB ]
 >>
 /TilingType 1
 /XStep 1
@@ -802,37 +717,11 @@ $patdef
 endstream
 endobj
 PATTERNDEF
-
-	return $xid;
-}
-
-sub fillPattern
-{
-	return $_[0]-> SUPER::fillPattern unless $#_;
-	$_[0]-> SUPER::fillPattern( $_[1]);
-	return unless $_[0]-> {can_draw};
-
-	my $self = $_[0];
-	my $fp   = $self-> SUPER::fillPattern;
-	my ($solidBack, $solidFore) = (0,0);
-	my $xid;
-	if( (ref($fp) // '') eq 'ARRAY') {
-		$solidBack = fp::is_empty($fp);
-		$solidFore = fp::is_solid($fp);
-		if ( !$solidBack && !$solidFore) {
-			my $fpid = join( '', map { sprintf("%02x", $_)} @$fp);
-			$xid  = $self-> emit_pattern($fpid, $fp);
+			$self-> {fp_hash}-> {$fpid} = $xid;
+		} else {
+			$xid = $self-> {fp_hash}-> {$fpid};
 		}
-	} elsif ( UNIVERSAL::isa($fp, 'Prima::Image')) {
-		my $fpid = "$fp";
-		$fpid =~ s/^Prima::(Image|Icon)=HASH\((.*)\)/$2/; # most common case
-		$fpid =~ s/(\W)/sprintf("%02x", ord($1))/ge;      # just in case
-		$fpid = (( $fp->type == im::BW ) ? "Mono_" : "Color_") . $fpid;
-
-		my $icon    = $fp->isa('Prima::Icon'); # XXX
-		$xid  = $self-> emit_pattern($fpid, $fp);
-	} else {
-		$solidFore = 1;
+		$self->{page_patterns}->{$xid}++;
 	}
 	$self-> {fpType} = $solidBack ? 'B' : ( $solidFore ? 'F' : $xid);
 	$self-> {changed}-> {fill} = 1;
@@ -845,31 +734,69 @@ sub compress
 	$self-> {compress} = $_[1];
 }
 
-sub graphic_context_push
-{
-	my $self = shift;
-	return 0 unless $self->SUPER::graphic_context_push;
-	$self->emit_content('q');
-	$self->{lock_margins}++;
-	return 1;
-}
+our $PI = 3.14159265358979323846264338327950288419716939937510;
+our $RAD = 180.0 / $PI;
 
-sub graphic_context_pop
+# L.Maisonobe 2003
+# http://www.spaceroots.org/documents/ellipse/elliptical-arc.pdf
+sub arc2cubics
 {
-	my $self = shift;
-	$self->emit_content('Q');
-	return unless $self->SUPER::graphic_context_pop;
-	$self->{lock_margins}--;
-	return 1;
+	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
+
+	my ($reverse, @out);
+	($start, $end, $reverse) = ( $end, $start, 1 ) if $start > $end;
+
+	push @out, $start;
+	# see defects appearing after 45 degrees:
+	# https://pomax.github.io/bezierinfo/#circles_cubic
+	while (1) {
+		if ( $end - $start > 45 ) {
+			push @out, $start += 45;
+			$start += 45;
+		} else {
+			push @out, $end;
+			last;
+		}
+	}
+	@out = map { $_ / $RAD } @out;
+
+	my $rx = $dx / 2;
+	my $ry = $dy / 2;
+
+	my @cubics;
+	for ( my $i = 0; $i < $#out; $i++) {
+		my ( $a1, $a2 ) = @out[$i,$i+1];
+		my $b           = $a2 - $a1;
+		my ( $sin1, $cos1, $sin2, $cos2) = ( sin($a1), cos($a1), sin($a2), cos($a2) );
+		my @d1  = ( -$rx * $sin1, -$ry * $cos1 );
+		my @d2  = ( -$rx * $sin2, -$ry * $cos2 );
+		my $tan = sin( $b / 2 ) / cos( $b / 2 );
+		my $a   = sin( $b ) * (sqrt( 4 + 3 * $tan * $tan) - 1) / 3;
+		my @p1  = ( $rx * $cos1, $ry * $sin1 );
+		my @p2  = ( $rx * $cos2, $ry * $sin2 );
+		my @points = (
+			@p1,
+			$p1[0] + $a * $d1[0],
+			$p1[1] - $a * $d1[1],
+			$p2[0] - $a * $d2[0],
+			$p2[1] + $a * $d2[1],
+			@p2
+		);
+		$points[$_] += $x for 0,2,4,6;
+		$points[$_] += $y for 1,3,5,7;
+		@points[0,1,2,3,4,5,6,7] = @points[6,7,4,5,2,3,0,1] if $reverse;
+		push @cubics, \@points;
+	}
+	return \@cubics;
 }
 
 sub arc
 {
 	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
-	return $self->primitive( arc => $x, $y, $dx, $dy, $start, $end) if $self-> is_custom_line;
+
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 
-	my $cubics  = Prima::Drawable::Path->arc2cubics($x, $y, $dx, $dy, $start, $end);
+	my $cubics  = $self-> arc2cubics($x, $y, $dx, $dy, $start, $end);
 	my $content = "@{ $cubics->[0] }[0,1] m\n";
 	$content   .= "@{$_}[2..7] c\n" for @$cubics;
 	$self-> stroke( $content . " S");
@@ -878,10 +805,10 @@ sub arc
 sub chord
 {
 	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
-	return $self->primitive( chord => $x, $y, $dx, $dy, $start, $end) if $self-> is_custom_line(1);
+
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 
-	my $cubics  = Prima::Drawable::Path->arc2cubics($x, $y, $dx, $dy, $start, $end);
+	my $cubics  = $self-> arc2cubics($x, $y, $dx, $dy, $start, $end);
 	my $content = "@{ $cubics->[0] }[0,1] m\n";
 	$content   .= "@{$_}[2..7] c\n" for @$cubics;
 	$self-> stroke( $content . " h S");
@@ -890,10 +817,9 @@ sub chord
 sub ellipse
 {
 	my ( $self, $x, $y, $dx, $dy) = @_;
-	return $self->primitive( ellipse => $x, $y, $dx, $dy) if $self-> is_custom_line(1);
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 
-	my $cubics  = Prima::Drawable::Path->arc2cubics($x, $y, $dx, $dy, 0, 360);
+	my $cubics  = $self-> arc2cubics($x, $y, $dx, $dy, 0, 360);
 	my $content = "@{ $cubics->[0] }[0,1] m\n";
 	$content   .= "@{$_}[2..7] c\n" for @$cubics;
 	$self-> stroke( $content . " h S");
@@ -902,9 +828,10 @@ sub ellipse
 sub fill_chord
 {
 	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
+
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 
-	my $cubics  = Prima::Drawable::Path->arc2cubics($x, $y, $dx, $dy, $start, $end);
+	my $cubics  = $self-> arc2cubics($x, $y, $dx, $dy, $start, $end);
 	my $content = "@{ $cubics->[0] }[0,1] m\n";
 	$content   .= "@{$_}[2..7] c\n" for @$cubics;
 	my $F = (($self-> fillMode & fm::Winding) == fm::Alternate) ? 'f*' : 'f';
@@ -914,9 +841,7 @@ sub fill_chord
 sub fill_ellipse
 {
 	my ( $self, $x, $y, $dx, $dy) = @_;
-	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
-
-	my $cubics  = Prima::Drawable::Path->arc2cubics($x, $y, $dx, $dy, 0, 360);
+	my $cubics  = $self-> arc2cubics($x, $y, $dx, $dy, 0, 360);
 	my $content = "@{ $cubics->[0] }[0,1] m\n";
 	$content   .= "@{$_}[2..7] c\n" for @$cubics;
 	$self-> stroke( $content . " h f");
@@ -925,10 +850,9 @@ sub fill_ellipse
 sub sector
 {
 	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
-	return $self->primitive( sector => $x, $y, $dx, $dy, $start, $end) if $self-> is_custom_line(1);
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 
-	my $cubics  = Prima::Drawable::Path->arc2cubics($x, $y, $dx, $dy, $start, $end);
+	my $cubics  = $self-> arc2cubics($x, $y, $dx, $dy, $start, $end);
 	my $content = "$x $y m @{ $cubics->[0] }[0,1] l\n";
 	$content   .= "@{$_}[2..7] c\n" for @$cubics;
 	$self-> stroke( $content . " h S");
@@ -939,7 +863,7 @@ sub fill_sector
 	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 
-	my $cubics  = Prima::Drawable::Path->arc2cubics($x, $y, $dx, $dy, $start, $end);
+	my $cubics  = $self-> arc2cubics($x, $y, $dx, $dy, $start, $end);
 	my $content = "$x $y m @{ $cubics->[0] }[0,1] l\n";
 	$content   .= "@{$_}[2..7] c" for @$cubics;
 	my $F = (($self-> fillMode & fm::Winding) == fm::Alternate) ? 'f*' : 'f';
@@ -955,7 +879,7 @@ sub text_out_outline
 
 sub glyph_out_outline
 {
-	my ( $self, $text, $from, $len) = @_;
+	my ( $self, $text, $from, $len ) = @_;
 
 	my $glyphs     = $text-> glyphs;
 	my $indexes    = $text-> indexes;
@@ -964,8 +888,9 @@ sub glyph_out_outline
 	my $fonts      = $text-> fonts;
 	my $plaintext  = $text-> [Prima::Drawable::Glyphs::CUSTOM()];
 	my @ix_lengths = defined($plaintext) ? $text-> index_lengths : ();
+	my $adv        = 0;
 	my $canvas     = $self->glyph_canvas;
-	my $resolution = 72.0 / $self->{resolution}->[0];
+	my $resolution = 72.27 / $self->{resolution}->[0];
 	my $keeper     = $self->{glyph_keeper};
 	my $font       = $self->{glyph_font};
 	my $div        = $self->{font_scale};
@@ -974,10 +899,13 @@ sub glyph_out_outline
 	$len += $from;
 	my $emit = '';
 	my $fid  = 0;
+	my $ff = $canvas->font;
 	my $curr_subfont = -1;
-	my @multi_tj;
+	my ($x, $y) = (0,0);
 	for ( my $i = $from; $i < $len; $i++) {
+		my $advance;
 		my $glyph     = $glyphs->[$i];
+		my ($x2, $y2) = ($adv, 0);
 		my $nfid = $fonts ? $fonts->[$i] : 0;
 		if ( $nfid != $fid ) {
 			my $newfont;
@@ -985,7 +913,7 @@ sub glyph_out_outline
 				$newfont = $self->{font};
 				$restore_font = 0;
 			} else {
-				my $src  = $self->font_mapper->get($nfid);
+				my $src  = $self-> fontMapperPalette($nfid);
 				my $dst  = \%{$self->{font}};
 				$newfont = Prima::Drawable->font_match( $src, $dst );
 				$restore_font = 1;
@@ -1003,17 +931,26 @@ sub glyph_out_outline
 			$curr_subfont = $subfont;
 			my $xid = $self-> {all_fonts}-> {$font}-> {xids}-> [ $subfont ] //= $self->new_dummy_obj;
 			$self->{page_fonts}->{$xid} //= 1;
-			if ( @multi_tj ) {
-				$emit .= "(".join('',@multi_tj).") Tj\n";
-				@multi_tj = ();
-			}
 			$emit .= "/F$xid $self->{font}->{size} Tf\n";
 		}
-		push @multi_tj, sprintf("\\%03o", $gid) if defined $gid;
-	}
-	if ( @multi_tj ) {
-		$emit .= "(".join('',@multi_tj).") Tj\n";
-		@multi_tj = ();
+		if ( $advances) {
+			$advance = $advances->[$i];
+			$x2 += $positions->[$i*2];
+			$y2 += $positions->[$i*2 + 1];
+		} else {
+			my $xr = $canvas->get_font_abc($glyph, $glyph, to::Glyphs);
+			$advance = ($$xr[0] + $$xr[1] + $$xr[2]) * $div;
+		}
+		$adv += $advance;
+		($x2, $y2) = map { int( $_ * 100 + 0.5) / 100 } $self->pixel2point($x2, $y2);
+		my $dx = $x2 - $x;
+		my $dy = $y2 - $y;
+		if  ($dx != 0 || $dy != 0) {
+			($dx, $dy) = map { int( $_ * 100 + 0.5) / 100 } ($dx, $dy);
+			$emit .= "$dx $dy Td ";
+		}
+		($x, $y) = ($x2, $y2);
+		$emit .= sprintf "<%02x> Tj\n", $gid if defined $gid;
 	}
 
 	if ($restore_font) {
@@ -1048,15 +985,13 @@ sub text_out
 	$self-> emit_content("q");
 	my $wmul = $self-> {font_x_scale};
 	if ( $self-> {font}-> {direction} != 0) {
-		my $r = $self-> {font}-> {direction} / $Prima::matrix::RAD;
+		my $r = $self-> {font}-> {direction};
 		my $sin1 = sin($r);
 		my $cos  = cos($r);
 		my $wcos = cos($r) * $wmul;
 		my $sin2 = -$sin1;
-		float_inplace($wcos, $sin1, $sin2, $cos, $x, $y);
 		$self-> emit_content("$wcos $sin1 $sin2 $cos $x $y cm");
 	} else {
-		float_inplace($wmul, $x, $y);
 		$self-> emit_content("$wmul 0 0 1 $x $y cm");
 	}
 
@@ -1065,12 +1000,12 @@ sub text_out
 		my ( $ds, $bs) = ( $self-> {font}-> {direction}, $self-> textOutBaseline);
 		$self-> {font}-> {direction} = 0;
 		$self-> textOutBaseline(1) unless $bs;
-		@rb = float_format($self-> pixel2point( @{$self-> get_text_box( $text, $from, $len)}));
+		@rb = $self-> pixel2point( @{$self-> get_text_box( $text, $from, $len)});
 		$self-> {font}-> {direction} = $ds;
 		$self-> textOutBaseline($bs) unless $bs;
 	}
 	if ( $self-> textOpaque) {
-		$self-> emit_content( lc $self-> cmd_rgb( $self-> backColor));
+		$self-> emit_content( uc $self-> cmd_rgb( $self-> backColor));
 		$self-> emit_content( "h @rb[0,1] m @rb[2,3] l @rb[6,7] l @rb[4,5] l f");
 	}
 
@@ -1104,7 +1039,6 @@ sub text_out
 sub rectangle
 {
 	my ( $self, $x1, $y1, $x2, $y2) = @_;
-	return $self->primitive( rectangle => $x1, $y1, $x2, $y2) if $self-> is_custom_line(1);
 	( $x1, $y1, $x2, $y2) = $self-> pixel2point( $x1, $y1, $x2, $y2);
 	$x2 -= $x1;
 	$y2 -= $y1;
@@ -1131,7 +1065,7 @@ sub bars
 	for ( $i = 0; $i < $c; $i += 4) {
 		$z .= "h @a[$i,$i+1] " . ($a[$i+2] - $a[$i]) . ' ' . ($a[$i+3] - $a[$i+1]) . " re f\n";
 	}
-	$self-> fill( $z);
+	$self-> stroke( $z);
 }
 
 sub clear
@@ -1147,7 +1081,6 @@ sub clear
 	$x2 -= $x1;
 	$y2 -= $y1;
 	my $c = lc $self-> cmd_rgb( $self-> backColor);
-	float_inplace($x1, $y1, $x2, $y2);
 	$self-> emit_content(<<CLEAR);
 $c
 h $x1 $y1 $x2 $y2 re f
@@ -1158,18 +1091,16 @@ CLEAR
 sub line
 {
 	my ( $self, $x1, $y1, $x2, $y2) = @_;
-	return $self->primitive(line => $x1, $y1, $x2, $y2) if $self-> is_custom_line;
-	( $x1, $y1, $x2, $y2) = float_format($self-> pixel2point( $x1, $y1, $x2, $y2));
+	( $x1, $y1, $x2, $y2) = $self-> pixel2point( $x1, $y1, $x2, $y2);
 	$self-> stroke("h $x1 $y1 m $x2 $y2 l S");
 }
 
 sub lines
 {
 	my ( $self, $array) = @_;
-	return $self->primitive( lines => $array ) if $self->is_custom_line;
 	my $i;
 	my $c = scalar @$array;
-	my @a = float_format($self-> pixel2point( @$array));
+	my @a = $self-> pixel2point( @$array);
 	$c = int( $c / 4) * 4;
 	my $z = '';
 	for ( $i = 0; $i < $c; $i += 4) {
@@ -1181,7 +1112,6 @@ sub lines
 sub polyline
 {
 	my ( $self, $array) = @_;
-	return $self->primitive( polyline => $array ) if $self->is_custom_line;
 	my $i;
 	my $c = scalar @$array;
 	my @a = $self-> pixel2point( @$array);
@@ -1218,7 +1148,7 @@ sub pixel
 	return cl::Invalid unless defined $pix;
 	my $c = lc $self-> cmd_rgb( $pix);
 	my $w;
-	($x, $y, $w) = float_format($self-> pixel2point( $x, $y, 1));
+	($x, $y, $w) = $self-> pixel2point( $x, $y, 1);
 	$self-> emit_content(<<PIXEL);
 q
 $c
@@ -1239,11 +1169,44 @@ sub put_image_indirect
 {
 	return 0 unless $_[0]-> {can_draw};
 	my ( $self, $image, $x, $y, $xFrom, $yFrom, $xDestLen, $yDestLen, $xLen, $yLen, $rop) = @_;
-	$rop //= rop::Default;
-	$rop = $image->get_effective_rop($rop);
 	return 1 if $rop == rop::NoOper;
 
-	$image = $self->prepare_image($image, $xFrom, $yFrom, $xLen, $yLen);
+	my $touch;
+	$touch = 1, $image = $image-> image if $image-> isa('Prima::DeviceBitmap');
+
+	unless ( $xFrom == 0 && $yFrom == 0 && $xLen == $image-> width && $yLen == $image-> height) {
+		$image = $image-> extract( $xFrom, $yFrom, $xLen, $yLen);
+		$touch = 1;
+	}
+
+	my $ib = $image-> get_bpp;
+	if ( $ib != $self-> get_bpp) {
+		$image = $image-> dup unless $touch;
+		if ( $self-> {grayscale} || $image-> type & im::GrayScale) {
+			$image-> type( im::Byte);
+		} else {
+			$image-> type( im::RGB);
+		}
+		$touch = 1;
+	} elsif ( $self-> {grayscale} || $image-> type & im::GrayScale) {
+		$image = $image-> dup unless $touch;
+		$image-> type( im::Byte);
+		$touch = 1;
+	}
+
+	$ib = $image-> get_bpp;
+	if ($ib != 8 && $ib != 24) {
+		$image = $image-> dup unless $touch;
+		$image-> type( im::RGB);
+		$touch = 1;
+	}
+
+	if ( $image-> type == im::RGB ) {
+		# invert BGR -> RGB
+		$image = $image-> dup unless $touch;
+		$image-> set(data => $image->data, type => im::fmtBGR | im::RGB);
+		$touch = 1;
+	}
 
 	my @is = $image-> size;
 	($x, $y, $xDestLen, $yDestLen) = $self-> pixel2point( $x, $y, $xDestLen, $yDestLen);
@@ -1255,6 +1218,11 @@ sub put_image_indirect
 	my $xid2;
 	my $mask = '';
 	if ( $image-> isa('Prima::Icon')) {
+		if ( $image-> maskType != 1 && $image-> maskType != 8) {
+			$image = $image-> dup unless $touch;
+			$image-> set(maskType => 1);
+			$touch = 1;
+		}
 		my $obj;
 		($xid2, $obj) = $self-> new_file_obj;
 		my $g  = $image-> mask;
@@ -1348,39 +1316,30 @@ sub new_path
 	return Prima::PS::PDF::Path->new(@_);
 }
 
-sub region
+package Prima::PS::PDF::Path;
+use base qw(Prima::Drawable::Path);
+
+sub entries
 {
-	return $_[0]->{region} unless $#_;
-	my ( $self, $region ) = @_;
-	if ( $region && !UNIVERSAL::isa($region, "Prima::PS::PDF::Region")) {
-		if ( UNIVERSAL::isa($region, 'Prima::Region')) {
-			$region = Prima::PS::PDF::Region-> new_from_region( $region, $self );
-		} else {
-			warn "Region is not a Prima::PS::PDF::Region";
-			return undef;
+	my $self = shift;
+	unless ( $self->{entries} ) {
+		local $self->{stack} = [];
+		local $self->{curr}  = { matrix => [ $self-> identity ] };
+		my $c = $self->{commands};
+		$self-> {entries} = [];
+		for ( my $i = 0; $i < @$c; ) {
+			my ($cmd,$len) = @$c[$i,$i+1];
+			$self-> can("_$cmd")-> ( $self, @$c[$i+2..$i+$len+1] );
+			$i += $len + 2;
 		}
+		$self->{last_matrix} = $self->{curr}->{matrix};
 	}
-	$self->{clipRect} = [0,0,0,0];
-	$self->{region} = $region;
-	$self-> change_transform;
+	return $self-> {entries};
 }
 
-package
-	Prima::PS::PDF::Path;
-use base qw(Prima::PS::Drawable::Path);
+sub emit { push @{shift->{entries}}, join(' ', @_) }
 
-my %dict = (
-	newpath   => '',
-	lineto    => 'l',
-	moveto    => 'm',
-	curveto   => 'c',
-	stroke    => 'S',
-	closepath => 'h',
-	fill_alt  => 'f*',
-	fill_wind => 'f',
-);
-
-sub dict { \%dict }
+sub last_point { @{$_[0]->{last_point} // [0,0]} }
 
 sub set_current_point
 {
@@ -1389,58 +1348,104 @@ sub set_current_point
 	$self-> {move_is_line} = 1;
 }
 
-sub region
-{
-	my ($self, $mode) = @_;
-	my $path = join "\n", @{$self-> entries};
-	$path .= ' h' unless $path =~ /h$/;
-	$path .= ' W';
-	$path .= '*' if ($mode // fm::Winding) & fm::Alternate;
-	return Prima::PS::PDF::Region->new( $path );
-}
-
-package
-	Prima::PS::PDF::Region;
-use base qw(Prima::PS::Drawable::Region);
-use Prima::PS::Format;
-
-sub other { UNIVERSAL::isa($_[0], "Prima::PS::PDF::Region") ? $_[0] : () }
-
-sub equals
+sub _open
 {
 	my $self = shift;
-	my $other = other(shift) or return;
-	return $self->{path} eq $other->{path};
+	$self-> {move_is_line} = 0;
+	$self->emit('')
 }
 
-sub combine
+sub _close     { shift->emit('h') }
+
+sub  _moveto
+{
+	my ( $self, $mx, $my, $rel) = @_;
+	($mx, $my) = $self-> canvas-> pixel2point( $mx, $my );
+	($mx, $my) = $self->matrix_apply($mx, $my);
+	my ($lx, $ly) = $rel ? $self->last_point : (0,0);
+	$lx += $mx;
+	$ly += $my;
+	@{$self-> {last_point}} = ($lx, $ly);
+	$self-> emit($lx, $ly, 'm');
+}
+
+sub _line
+{
+	my ( $self, $line ) = @_;
+	my @line = $self-> canvas-> pixel2point( @$line );
+	@line = @{ $self-> matrix_apply( \@line ) };
+	$self-> set_current_point( shift @line, shift @line );
+	@{$self-> {last_point}} = @line[-2,-1];
+	for ( my $i = 0; $i < @line; $i += 2 ) {
+		$self->emit(@line[$i,$i+1], 'l');
+	}
+}
+
+sub conic2curve
+{
+	my ($x0, $y0, $x1, $y1, $x2, $y2) = @_;
+	my (@cp1, @cp2);
+	$cp1[0] = $x0 + 2 / 3 * ($x1 - $x0);
+	$cp1[1] = $y0 + 2 / 3 * ($y1 - $y0);
+	$cp2[0] = $x2 + 2 / 3 * ($x1 - $x2);
+	$cp2[1] = $y2 + 2 / 3 * ($y1 - $y2);
+	return @cp1, @cp2, $x2, $y2;
+}
+
+sub _spline
+{
+	my ( $self, $points, $options ) = @_;
+	my @p = $self-> canvas-> pixel2point( @$points );
+	@p = @{ $self-> matrix_apply( \@p ) };
+
+	$options->{degree} //= 2;
+	return if $options->{degree} > 3;
+	my @p0 = @p[0,1];
+	$self-> set_current_point( @p0 );
+	if ( $options->{degree} == 2 ) {
+		for ( my $i = 2; $i < @p; $i += 4 ) {
+			my @pp = conic2curve( @p0, @p[$i .. $i + 3] );
+			$self->emit(@pp, 'c');
+			@p0 = @pp[-2,-1];
+		}
+	} else {
+		for ( my $i = 2; $i < @p; $i += 4 ) {
+			my @pp = @p[$i .. $i + 5];
+			$self->emit(@pp, 'c');
+		}
+	}
+}
+
+sub _arc
+{
+	my ( $self, $from, $to, $rel ) = @_;
+	my $cubics = $self->canvas->arc2cubics( 0, 0, 1, 1, $from, $to);
+
+	if ( $rel ) {
+		my ($lx,$ly) = $self->last_point;
+		my $pts = $cubics->[0];
+		my $m = $self->{curr}->{matrix};
+		my @s = $self->matrix_apply( $pts->[0], $pts->[1]);
+		$m->[4] += $lx - $s[0];
+		$m->[5] += $ly - $s[1];
+	}
+	my @p = map { $self-> matrix_apply( $_ ) } @$cubics;
+	$self-> set_current_point( @{$p[0]}[0,1] );
+	$self-> emit( @{$_}[2..7], 'c') for @p;
+}
+
+sub stroke
 {
 	my $self = shift;
-	my $other = other(shift) or return;
-	$self->{path} .= "\n" . $other->apply_offset;
+	$self-> canvas-> stroke( join("\n", @{ $self->entries }, 'S' ));
 }
 
-sub is_empty { shift->{path} !~ /[Sf]/ }
-
-sub apply_offset
+sub fill
 {
-	my ($self, $canvas) = @_;
-	my $path = $self->apply_offset;
-	$path =~ s/\bn\b//;
-	return $path;
-}
-
-sub apply_clip
-{
-	my ($self, $canvas) = @_;
-	my $path = $self->{path};
-	my @offset = @{ $self->{offset} };
-	return "$path n" if 0 == grep { $_ != 0 } @offset;
-
-	@offset = $canvas-> pixel2point(@offset) if $canvas;
-	float_inplace(@offset);
-
-	return "1 0 0 1 @offset cm $path n 1 0 0 1 -$offset[0] -$offset[1] cm";
+	my ( $self, $fillMode ) = @_;
+	$fillMode //= $self->canvas->fillMode;
+	$fillMode = ((($fillMode & fm::Winding) == fm::Alternate) ? 'f*' : 'f');
+	$self-> canvas-> fill( join("\n", @{ $self->entries }, $fillMode ));
 }
 
 1;
@@ -1453,27 +1458,15 @@ Prima::PS::PDF -  PDF interface to Prima::Drawable
 
 =head1 SYNOPSIS
 
-Recommended usage:
-
-	use Prima::PS::Printer;
-	my $x = Prima::PS::PDF::File-> new( file => 'out.pdf');
-
-or
-
-	my $x = Prima::PS::PDF::FileHandle-> new( handle => \&STDOUT);
-
-Low-level:
-
+	use Prima;
 	use Prima::PS::PDF;
+
 	my $x = Prima::PS::PDF-> create( onSpool => sub {
 		open F, ">> ./test.pdf";
 		binmode F;
 		print F $_[1];
 		close F;
 	});
-
-Printing:
-
 	die "error:$@" unless $x-> begin_doc;
 	$x-> font-> size( 30);
 	$x-> text_out( "hello!", 100, 100);
@@ -1482,9 +1475,9 @@ Printing:
 
 =head1 DESCRIPTION
 
-Implements the Prima library interface to PDF v1.4.
-The module is designed to be compliant with the Prima::Drawable interface.
-All properties' behavior is as same as Prima::Drawable's except those
+Realizes the Prima library interface to PDF v1.4.
+The module is designed to be compliant with Prima::Drawable interface.
+All properties' behavior is as same as Prima::Drawable's, except those
 described below.
 
 =head2 Inherited properties
@@ -1493,9 +1486,13 @@ described below.
 
 =item ::resolution
 
-Can be set while the object is in the normal stage - cannot be changed if the
-document is opened. Applies to implementation of the fillPattern property and
-general pixel-to-point and vice versa calculations
+Can be set while object is in normal stage - cannot be changed if document
+is opened. Applies to fillPattern realization and general pixel-to-point
+and vice versa calculations
+
+=item ::region
+
+- ::region is not realized ( yet?)
 
 =back
 
@@ -1503,23 +1500,31 @@ general pixel-to-point and vice versa calculations
 
 =over
 
-=item grayscale
+=item ::grayscale
 
 could be 0 or 1
 
-=item pageSize
+=item ::pageSize
 
-The physical page dimension, in points
+physical page dimension, in points
 
-=item pageMargins
+=item ::pageMargins
 
-Non-printable page area, an array of 4 integers:
-left, bottom, right, and top margins in points.
+non-printable page area, an array of 4 integers:
+left, bottom, right and top margins in points.
 
-=item reversed
+=item ::reversed
 
 if 1, a 90 degrees rotated document layout is assumed
 
+=item ::rotate and ::scale
+
+along with Prima::Drawable::translate provide PS-specific
+transformation matrix manipulations. ::rotate is number,
+measured in degrees, counter-clockwise. ::scale is array of
+two numbers, respectively x- and y-scale. 1 is 100%, 2 is 200%
+etc.
+
 =back
 
 =head2 Internal methods
@@ -1532,16 +1537,15 @@ Helpers for translation from pixel to points and vice versa.
 
 =item spool
 
-Prima::PS::PDF is not responsible for the output of the generated
-document, it only calls the C<::spool> method when the document is closed
-through an C<::end_doc> call. By default discards the data.  The
-C<Prima::PS::Printer> class handles the spooling logic.
+Prima::PS::Drawable is not responsible for output of
+generated document, it just calls ::spool when document
+is closed through ::end_doc. By default just skips data.
+Prima::PS::Printer handles spooling logic.
 
 =item fonts
 
-Calls Prima::Application::fonts and returns its output filtered so that only
-the fonts that support the C<iso10646-1> encoding are present. This effectively
-allows only unicode output.
+Returns Prima::Application::fonts, however with C<iso10646-1> encoding only.
+That effectively allows only unicode output.
 
 =back
 
