@@ -2,7 +2,6 @@ package Prima::PS::PostScript;
 use strict;
 use warnings;
 use Prima;
-use Prima::PS::Format;
 use Prima::PS::Type1;
 use Prima::PS::TempFile;
 use base qw(Prima::PS::Drawable);
@@ -87,41 +86,33 @@ sub change_transform
 	my ( $self, $gsave ) = @_;
 	return if $self-> {delay};
 
-	my ($doClip, @cr);
-	my $rg = $self-> region;
-	unless ( $rg ) {
-		@cr   = $self-> clipRect;
-		$cr[2]  -= $cr[0];
-		$cr[3]  -= $cr[1];
-		$doClip  = grep { $_ != 0 } @cr;
-	}
+	my @tp = $self-> translate;
+	my @cr = $self-> clipRect;
+	my @sc = $self-> scale;
+	my $ro = $self-> rotate;
+	$cr[2] -= $cr[0];
+	$cr[3] -= $cr[1];
+	my $doClip = grep { $_ != 0 } @cr;
+	my $doTR   = grep { $_ != 0 } @tp;
+	my $doSC   = grep { $_ != 0 } @sc;
 
-	my $m = $self-> get_matrix;
-	my $doMx = !$m->is_identity;
-
-	if ( !$doClip && !$doMx && !$rg) {
+	if ( !$doClip && !$doTR && !$doSC && !$ro) {
 		$self-> emit(':') if $gsave;
 		return;
 	}
 
+	@cr = $self-> pixel2point( @cr);
+	@tp = $self-> pixel2point( @tp);
+	my $mcr3 = -$cr[3];
+
 	$self-> emit(';') unless $gsave;
 	$self-> emit(':');
-
-	if ( $rg ) {
-		$self-> emit($rg-> apply_clip($self) . ' C');
-	} elsif ( $doClip ) {
-		@cr = $self-> pixel2point( @cr);
-		float_inplace(@cr);
-		$self-> emit("N $cr[0] $cr[1] M 0 $cr[3] L $cr[2] 0 L 0 -$cr[3] L X C");
-	}
-
-	if ( $doMx ) {
-		my @tm = $self-> pixel2point( @$m[4,5] );
-		my @xm = @$m[0..3];
-		float_inplace(@xm, @tm);
-		$self-> emit("[@xm @tm] CM");
-	}
-
+	$self-> emit(<<CLIP) if $doClip;
+N $cr[0] $cr[1] M 0 $cr[3] L $cr[2] 0 L 0 $mcr3 L X C
+CLIP
+	$self-> emit("@tp T") if $doTR;
+	$self-> emit("@sc Z") if $doSC;
+	$self-> emit("$ro R") if $ro != 0;
 	$self-> {changed}-> {$_} = 1 for qw(fill linePattern lineWidth lineJoin lineEnd miterLimit font);
 }
 
@@ -133,7 +124,7 @@ sub fill
 		$r1 == rop::NoOper &&
 		$r2 == rop::NoOper;
 
-	if ( $r2 != rop::NoOper && $self-> {fpType} ne 'F' && $self->{fpType} !~ /^Color/ ) {
+	if ( $r2 != rop::NoOper && $self-> {fpType} ne 'F') {
 		my $bk =
 			( $r2 == rop::Blackness) ? 0 :
 			( $r2 == rop::Whiteness) ? 0xffffff : $self-> backColor;
@@ -149,10 +140,6 @@ sub fill
 		if ($self-> {changed}-> {fill}) {
 			if ( $self-> {fpType} eq 'F') {
 				$self-> emit( $self-> cmd_rgb( $c));
-			} elsif ( $self->{fpType} =~ /^Color/ ) {
-					$self-> emit(<<IMGPAT);
-\/Pattern SS Pat_$self->{fpType} SC
-IMGPAT
 			} else {
 				my ( $r, $g, $b) = (
 					int((($c & 0xff0000) >> 16) * 100 / 256 + 0.5) / 100,
@@ -187,12 +174,10 @@ sub stroke
 		$r1 == rop::NoOper &&
 		$r2 == rop::NoOper;
 
-	my $lw_changed;
 	if ( $self-> {changed}-> {lineWidth}) {
 		my ($lw) = $self-> pixel2point($self-> lineWidth);
-		$self-> emit( float_format($lw) . ' SW');
+		$self-> emit( $lw . ' SW');
 		$self-> {changed}-> {lineWidth} = 0;
-		$lw_changed++;
 	}
 
 	if ( $self-> {changed}-> {lineEnd}) {
@@ -208,9 +193,9 @@ sub stroke
 		$self-> emit( "$id SJ");
 		$self-> {changed}-> {lineJoin} = 0;
 	}
-
+	
 	if ( $self-> {changed}-> {miterLimit}) {
-		my $ml = float_format($self-> miterLimit);
+		my $ml = $self-> miterLimit;
 		$self-> emit( "$ml ML");
 		$self-> {changed}-> {miterLimit} = 0;
 	}
@@ -232,20 +217,14 @@ sub stroke
 			( $r1 == rop::Blackness) ? 0 :
 			( $r1 == rop::Whiteness) ? 0xffffff : $self-> color;
 
-		if ( $self-> {changed}-> {linePattern} || $lw_changed ) {
+		if ( $self-> {changed}-> {linePattern}) {
 			if ( length( $lp) == 1) {
-				$self-> emit('[] 0 SD') if $self-> {changed}-> {linePattern};
+				$self-> emit('[] 0 SD');
 			} else {
-				my $lw = $self->lineWidth || 2.0; # because to be divided by 2
-				my @x = map { ord } split('', $lp);
+				my @x = split('', $lp);
 				push( @x, 0) if scalar(@x) % 1;
-				my @lp;
-				for ( my $i = 0; $i < @x; $i += 2 ) {
-					push @lp, ($x[$i] - 1) * $lw / 2;
-					push @lp, ($x[$i+1]) * $lw / 2;
-				}
-				@lp = map { $_ || 1 } @lp;
-				$self-> emit("[@lp] 0 SD");
+				@x = map { ord($_) } @x;
+				$self-> emit("[@x] 0 SD");
 			}
 			$self-> {changed}-> {linePattern} = 0;
 		}
@@ -263,7 +242,7 @@ sub stroke
 sub begin_doc
 {
 	my ( $self, $docName) = @_;
-	$@ = 'Bad object', return 0 if $self-> get_paint_state;
+	return 0 if $self-> get_paint_state;
 	$self-> {ps_data}  = '';
 	$self-> {can_draw} = 1;
 
@@ -318,12 +297,12 @@ $extras
 %%EndComments
 
 /d/def load def/,/load load d/~/exch , d/S/show , d/:/gsave , d/;/grestore ,
-d/N/newpath , d/M/moveto , d/L/rlineto , d/X/closepath , d/C/clip , d/U/curveto ,
+d/N/newpath , d/M/moveto , d/L/rlineto , d/X/closepath , d/C/clip ,
 d/T/translate , d/R/rotate , d/Y/glyphshow , d/P/showpage , d/Z/scale , d/I/imagemask ,
 d/@/dup , d/G/setgray , d/A/setrgbcolor , d/l/lineto , d/F/fill ,
 d/FF/findfont , d/XF/scalefont , d/SF/setfont ,
 d/O/stroke , d/SD/setdash , d/SL/setlinecap , d/SW/setlinewidth ,
-d/SJ/setlinejoin , d/E/eofill , d/ML/setmiterlimit , d/CM/concat ,
+d/SJ/setlinejoin , d/E/eofill , d/ML/setmiterlimit ,
 d/SS/setcolorspace , d/SC/setcolor , d/SM/setmatrix , d/SPD/setpagedevice ,
 d/SP/setpattern , d/CP/currentpoint , d/MX/matrix , d/MP/makepattern ,
 d/b/begin , d/e/end , d/t/true , d/f/false , d/?/ifelse , d/a/arc ,
@@ -346,7 +325,7 @@ PREFIX
 
 	$self-> {changed} = { map { $_ => 0 } qw(
 		fill lineEnd linePattern lineWidth lineJoin miterLimit font)};
-	$self-> {paint_state} = ps::Enabled;
+	$self-> SUPER::begin_paint;
 	$self-> save_state;
 
 	$self-> {delay} = 1;
@@ -365,8 +344,8 @@ sub abort_doc
 	my $self = $_[0];
 	return unless $self-> {can_draw};
 	$self-> {can_draw} = 0;
-	delete $self-> {paint_state};
-	$self-> abandon_state;
+	$self-> SUPER::end_paint;
+	$self-> restore_state;
 	delete $self-> {$_} for
 		qw (save_state ps_data changed page_prefix);
 }
@@ -374,7 +353,7 @@ sub abort_doc
 sub end_doc
 {
 	my $self = $_[0];
-	$@ = 'Bad object', return 0 unless $self-> {can_draw};
+	return 0 unless $self-> {can_draw};
 	$self-> {can_draw} = 0;
 
 	$self->{glyph_keeper}-> evacuate( sub { $self->spool( $_[0] ) } )
@@ -391,7 +370,7 @@ sub end_doc
 PSFOOTER
 
 	$self-> {can_draw} = 0;
-	delete $self-> {paint_state};
+	$self-> SUPER::end_paint;
 	$self-> restore_state;
 	delete $self-> {$_} for
 		qw (save_state changed ps_data page_prefix glyph_keeper glyph_font);
@@ -423,73 +402,6 @@ sub pages { $_[0]-> {pages} }
 
 # properties
 
-sub emit_pattern
-{
-	my ( $self, $fpid, $fp) = @_;
-
-	return if exists $self-> {fp_hash}-> {$fpid};
-	$self-> {fp_hash}-> {$fpid} = 1;
-
-	my (@sz, $imgdef, $imgcdef, $depth, $paint);
-	$imgdef = '';
-	if ( ref($fp) eq 'ARRAY') {
-		@sz      = (8,8);
-		$paint   = 2;
-		$imgcdef = 'I';
-		$depth   = 'f';
-		$imgdef  = join( '', map { sprintf("%02x", $_)} @$fp);
-	} else {
-		@sz = $fp->size;
-		my ( $ls, $stride, $data );
-		if ( $fp->type == im::BW ) {
-			$paint   = 2;
-			$depth   = 't';
-			$imgcdef = 'I';
-			$stride  = int($fp-> width / 8) +!!+ ($fp->width % 8);
-		} else {
-			$paint   = 1;
-			$depth   = '8';
-			$imgcdef = ($fp->type & im::GrayScale) ? 'image' : 'false 3 colorimage';
-			$fp      = $self->prepare_image($fp);
-			$stride  = $fp->get_bpp * $fp->width / 8;
-		}
-		$ls      = $fp->lineSize;
-		$data    = $fp->data;
-		for my $y ( 0 .. $fp->height - 1 ) {
-			$imgdef .= unpack("H*", substr($data, $y * $ls, $stride));
-		}
-	}
-
-	my @scaleto = $self-> pixel2point(@sz);
-
-	# PatternType 1 = Tiling pattern
-	# PaintType   1 = Colored
-	# PaintType   2 = Uncolored
-	$self-> emit( <<PATTERNDEF);
-:
-[1 0 0 1 0 0] SM
-<<
-\/PatternType 1
-\/PaintType $paint
-\/TilingType 1
-\/BBox [ 0 0 @scaleto]
-\/XStep $scaleto[0]
-\/YStep $scaleto[1]
-\/PaintProc { b
-:
-@scaleto Z
-@sz $depth
-[$sz[0] 0 0 $sz[0] 0 0]
-< $imgdef > $imgcdef
-;
-e
-} bind
->> MX MP
-\/Pat_$fpid ~ d
-;
-PATTERNDEF
-}
-
 sub fillPattern
 {
 	return $_[0]-> SUPER::fillPattern unless $#_;
@@ -497,28 +409,37 @@ sub fillPattern
 	return unless $_[0]-> {can_draw};
 
 	my $self = $_[0];
-	my $fp  = $self-> SUPER::fillPattern;
-	my ($solidBack, $solidFore) = (0,0);
+	my @fp  = @{$self-> SUPER::fillPattern};
+	my $solidBack = ! grep { $_ != 0 } @fp;
+	my $solidFore = ! grep { $_ != 0xff } @fp;
 	my $fpid;
+	my @scaleto = $self-> pixel2point( 8, 8);
+	if ( !$solidBack && !$solidFore) {
+		$fpid = join( '', map { sprintf("%02x", $_)} @fp);
+		unless ( exists $self-> {fp_hash}-> {$fpid}) {
+			$self-> emit( <<PATTERNDEF);
+<<
+\/PatternType 1 \% Tiling pattern
+\/PaintType 2 \% Uncolored
+\/TilingType 1
+\/BBox [ 0 0 @scaleto]
+\/XStep $scaleto[0]
+\/YStep $scaleto[1]
+\/PaintProc { b
+:
+@scaleto Z
+8 8 t
+[8 0 0 8 0 0]
+< $fpid > I
+;
+e
+} bind
+>> MX MP
+\/Pat_$fpid ~ d
 
-	if( (ref($fp) // '') eq 'ARRAY') {
-		$solidBack = fp::is_empty($fp);
-		$solidFore = fp::is_solid($fp);
-		my @scaleto = $self-> pixel2point( 8, 8);
-		if ( !$solidBack && !$solidFore) {
-			$fpid = join( '', map { sprintf("%02x", $_)} @$fp);
-			$self-> emit_pattern($fpid, $fp);
+PATTERNDEF
+			$self-> {fp_hash}-> {$fpid} = 1;
 		}
-	} elsif ( UNIVERSAL::isa($fp, 'Prima::Image')) {
-		$fpid = "$fp";
-		$fpid =~ s/^Prima::(Image|Icon)=HASH\((.*)\)/$2/; # most common case
-		$fpid =~ s/(\W)/sprintf("%02x", ord($1))/ge;      # just in case
-		$fpid = (( $fp->type == im::BW ) ? "Mono_" : "Color_") . $fpid;
-
-		my $icon    = $fp->isa('Prima::Icon'); # XXX
-		$self-> emit_pattern($fpid, $fp);
-	} else {
-		$solidFore = 1;
 	}
 	$self-> {fpType} = $solidBack ? 'B' : ( $solidFore ? 'F' : $fpid);
 	$self-> {changed}-> {fill} = 1;
@@ -538,28 +459,11 @@ sub pageDevice
 	$_[0]-> {pageDevice} = $_[1] unless $_[0]-> get_paint_state;
 }
 
-sub graphic_context_push
-{
-	my $self = shift;
-	return 0 unless $self->SUPER::graphic_context_push;
-	$self->emit(':');
-	return 1;
-}
-
-sub graphic_context_pop
-{
-	my $self = shift;
-	$self->emit(';');
-	return unless $self->SUPER::graphic_context_pop;
-	return 1;
-}
-
 # primitives
 
 sub arc
 {
 	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
-	return $self->primitive( arc => $x, $y, $dx, $dy, $start, $end) if $self-> is_custom_line;
 	my $try = $dy / $dx;
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 	my $rx = $dx / 2;
@@ -573,7 +477,6 @@ ARC
 sub chord
 {
 	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
-	return $self->primitive( chord => $x, $y, $dx, $dy, $start, $end) if $self-> is_custom_line(1);
 	my $try = $dy / $dx;
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 	my $rx = $dx / 2;
@@ -587,7 +490,6 @@ CHORD
 sub ellipse
 {
 	my ( $self, $x, $y, $dx, $dy) = @_;
-	return $self->primitive( ellipse => $x, $y, $dx, $dy) if $self-> is_custom_line(1);
 	my $try = $dy / $dx;
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 	my $rx = $dx / 2;
@@ -626,7 +528,6 @@ ELLIPSE
 sub sector
 {
 	my ( $self, $x, $y, $dx, $dy, $start, $end) = @_;
-	return $self->primitive( sector => $x, $y, $dx, $dy, $start, $end) if $self-> is_custom_line(1);
 	my $try = $dy / $dx;
 	( $x, $y, $dx, $dy) = $self-> pixel2point( $x, $y, $dx, $dy);
 	my $rx = $dx / 2;
@@ -671,7 +572,7 @@ sub glyph_out_outline
 	my @ix_lengths = defined($plaintext) ? $text-> index_lengths : ();
 	my $adv        = 0;
 	my $canvas     = $self->glyph_canvas;
-	my $resolution = 72.0 / $self->{resolution}->[0];
+	my $resolution = 72.27 / $self->{resolution}->[0];
 	my $keeper     = $self->{glyph_keeper};
 	my $font       = $self->{glyph_font};
 	my $div        = $self->{font_scale};
@@ -680,6 +581,7 @@ sub glyph_out_outline
 	$len += $from;
 	my $emit = '';
 	my $fid  = 0;
+	my $ff = $canvas->font;
 	for ( my $i = $from; $i < $len; $i++) {
 		my $advance;
 		my $glyph     = $glyphs->[$i];
@@ -691,7 +593,7 @@ sub glyph_out_outline
 				$newfont = $self->{font};
 				$restore_font = 0;
 			} else {
-				my $src  = $self->font_mapper->get($nfid);
+				my $src  = $self-> fontMapperPalette($nfid);
 				my $dst  = \%{$self->{font}};
 				$newfont = Prima::Drawable->font_match( $src, $dst );
 				$restore_font = 1;
@@ -704,10 +606,7 @@ sub glyph_out_outline
 		my $char = defined($plaintext) ?
 			substr( $plaintext, $indexes->[$i] & ~to::RTL, $ix_lengths[$i]) :
 			undef;
-		my $gid =
-			$keeper-> use_char($canvas, $font, $glyph, $char) //
-			$Prima::PS::Unicode->{$char} // # not a single vector font found
-			'question';
+		my $gid = $keeper-> use_char($canvas, $font, $glyph, $char);
 		if ( $advances) {
 			$advance = $advances->[$i];
 			$x2 += $positions->[$i*2];
@@ -717,8 +616,13 @@ sub glyph_out_outline
 			$advance = ($$xr[0] + $$xr[1] + $$xr[2]) * $div;
 		}
 		$adv += $advance;
-		($x2, $y2) = float_format($self->pixel2point($x2, $y2));
-		$emit .= "$x2 $y2 M " if $x2 != 0 || $y2 != 0;
+		if ( defined $gid ) {
+			($x2, $y2) = map { int( $_ * 100 + 0.5) / 100 } $self->pixel2point($x2, $y2);
+			$emit .= "$x2 $y2 M " if $x2 != 0 || $y2 != 0;
+		} else {
+			# not a single vector font found
+			$gid //= $Prima::PS::Unicode->{$char} // 'question';
+		}
 		$emit .= "/$gid Y\n";
 	}
 
@@ -772,7 +676,7 @@ sub text_out
 		my ( $ds, $bs) = ( $self-> {font}-> {direction}, $self-> textOutBaseline);
 		$self-> {font}-> {direction} = 0;
 		$self-> textOutBaseline(1) unless $bs;
-		@rb = float_format($self-> pixel2point( @{$self-> get_text_box( $text, $from, $len)}));
+		@rb = $self-> pixel2point( @{$self-> get_text_box( $text, $from, $len)});
 		$self-> {font}-> {direction} = $ds;
 		$self-> textOutBaseline($bs) unless $bs;
 	}
@@ -808,7 +712,7 @@ sub text_out
 sub bar
 {
 	my ( $self, $x1, $y1, $x2, $y2) = @_;
-	( $x1, $y1, $x2, $y2) = float_format($self-> pixel2point( $x1, $y1, $x2, $y2));
+	( $x1, $y1, $x2, $y2) = $self-> pixel2point( $x1, $y1, $x2, $y2);
 	$self-> fill( "N $x1 $y1 M $x1 $y2 l $x2 $y2 l $x2 $y1 l X F");
 }
 
@@ -817,7 +721,7 @@ sub bars
 	my ( $self, $array) = @_;
 	my $i;
 	my $c = scalar @$array;
-	my @a = float_format($self-> pixel2point( @$array));
+	my @a = $self-> pixel2point( @$array);
 	$c = int( $c / 4) * 4;
 	my $z = '';
 	for ( $i = 0; $i < $c; $i += 4) {
@@ -829,8 +733,7 @@ sub bars
 sub rectangle
 {
 	my ( $self, $x1, $y1, $x2, $y2) = @_;
-	return $self->primitive( rectangle => $x1, $y1, $x2, $y2) if $self-> is_custom_line(1);
-	( $x1, $y1, $x2, $y2) = float_format($self-> pixel2point( $x1, $y1, $x2, $y2));
+	( $x1, $y1, $x2, $y2) = $self-> pixel2point( $x1, $y1, $x2, $y2);
 	$self-> stroke( "N $x1 $y1 M $x1 $y2 l $x2 $y2 l $x2 $y1 l X O");
 }
 
@@ -843,7 +746,7 @@ sub clear
 			($x1, $y1, $x2, $y2) = (0,0,@{$self-> {size}});
 		}
 	}
-	( $x1, $y1, $x2, $y2) = float_format($self-> pixel2point( $x1, $y1, $x2, $y2));
+	( $x1, $y1, $x2, $y2) = $self-> pixel2point( $x1, $y1, $x2, $y2);
 	my $c = $self-> cmd_rgb( $self-> backColor);
 	$self-> emit(<<CLEAR);
 $c
@@ -855,18 +758,16 @@ CLEAR
 sub line
 {
 	my ( $self, $x1, $y1, $x2, $y2) = @_;
-	return $self->primitive( line => $x1, $y1, $x2, $y2) if $self-> is_custom_line;
-	( $x1, $y1, $x2, $y2) = float_format($self-> pixel2point( $x1, $y1, $x2, $y2));
+	( $x1, $y1, $x2, $y2) = $self-> pixel2point( $x1, $y1, $x2, $y2);
 	$self-> stroke("N $x1 $y1 M $x2 $y2 l O");
 }
 
 sub lines
 {
 	my ( $self, $array) = @_;
-	return $self->primitive( lines => $array ) if $self-> is_custom_line;
 	my $i;
 	my $c = scalar @$array;
-	my @a = float_format($self-> pixel2point( @$array));
+	my @a = $self-> pixel2point( @$array);
 	$c = int( $c / 4) * 4;
 	my $z = '';
 	for ( $i = 0; $i < $c; $i += 4) {
@@ -878,10 +779,9 @@ sub lines
 sub polyline
 {
 	my ( $self, $array) = @_;
-	return $self->primitive( polyline => $array ) if $self-> is_custom_line;
 	my $i;
 	my $c = scalar @$array;
-	my @a = float_format($self-> pixel2point( @$array));
+	my @a = $self-> pixel2point( @$array);
 	$c = int( $c / 2) * 2;
 	return if $c < 2;
 	my $z = "N @a[0,1] M ";
@@ -899,7 +799,7 @@ sub fillpoly
 	my $c = scalar @$array;
 	$c = int( $c / 2) * 2;
 	return if $c < 2;
-	my @a = float_format($self-> pixel2point( @$array));
+	my @a = $self-> pixel2point( @$array);
 	my $x = "N @a[0,1] M ";
 	for ( $i = 2; $i < $c; $i += 2) {
 		$x .= "@a[$i,$i+1] l ";
@@ -913,7 +813,7 @@ sub pixel
 	my ( $self, $x, $y, $pix) = @_;
 	return cl::Invalid unless defined $pix;
 	my $c = $self-> cmd_rgb( $pix);
-	($x, $y) = float_format($self-> pixel2point( $x, $y));
+	($x, $y) = $self-> pixel2point( $x, $y);
 	$self-> emit(<<PIXEL);
 :
 $c
@@ -926,10 +826,44 @@ PIXEL
 sub put_image_indirect
 {
 	return 0 unless $_[0]-> {can_draw};
-	my ( $self, $image, $x, $y, $xFrom, $yFrom, $xDestLen, $yDestLen, $xLen, $yLen, $rop) = @_;
-	$rop //= rop::Default;
-	$rop = $image->get_effective_rop($rop);
-	return 1 if $rop == rop::NoOper;
+	my ( $self, $image, $x, $y, $xFrom, $yFrom, $xDestLen, $yDestLen, $xLen, $yLen) = @_;
+
+	my $touch;
+	$touch = 1, $image = $image-> image if $image-> isa('Prima::DeviceBitmap');
+
+	unless ( $xFrom == 0 && $yFrom == 0 && $xLen == $image-> width && $yLen == $image-> height) {
+		$image = $image-> extract( $xFrom, $yFrom, $xLen, $yLen);
+		$touch = 1;
+	}
+
+	my $ib = $image-> get_bpp;
+	if ( $ib != $self-> get_bpp) {
+		$image = $image-> dup unless $touch;
+		if ( $self-> {grayscale} || $image-> type & im::GrayScale) {
+			$image-> type( im::Byte);
+		} else {
+			$image-> type( im::RGB);
+		}
+		$touch = 1;
+	} elsif ( $self-> {grayscale} || $image-> type & im::GrayScale) {
+		$image = $image-> dup unless $touch;
+		$image-> type( im::Byte);
+		$touch = 1;
+	}
+
+	$ib = $image-> get_bpp;
+	if ($ib != 8 && $ib != 24) {
+		$image = $image-> dup unless $touch;
+		$image-> type( im::RGB);
+		$touch = 1;
+	}
+
+	if ( $image-> type == im::RGB ) {
+		# invert BGR -> RGB
+		$image = $image-> dup unless $touch;
+		$image-> set(data => $image->data, type => im::fmtBGR | im::RGB);
+		$touch = 1;
+	}
 
 	my @is = $image-> size;
 	($x, $y, $xDestLen, $yDestLen) = $self-> pixel2point( $x, $y, $xDestLen, $yDestLen);
@@ -937,15 +871,13 @@ sub put_image_indirect
 		$is[0] / $xLen * $xDestLen,
 		$is[1] / $yLen * $yDestLen,
 	);
-	float_inplace($x, $y, @fullScale);
-	$self-> emit(": $x $y T @fullScale Z");
 
-	$image = $self->prepare_image($image, $xFrom, $yFrom, $xLen, $yLen);
 	my $g  = $image-> data;
 	my $bt = ( $image-> type & im::BPP) * $is[0] / 8;
 	my $ls = $image->lineSize;
 	my ( $i, $j);
 
+	$self-> emit(": $x $y T @fullScale Z");
 	$self-> emit("/scanline $bt string d");
 	$self-> emit("@is 8 [$is[0] 0 0 $is[1] 0 0]");
 	$self-> emit('{currentfile scanline readhexstring pop}');
@@ -970,101 +902,6 @@ sub apply_canvas_font
 	}
 }
 
-sub new_path
-{
-	return Prima::PS::PostScript::Path->new(@_);
-}
-
-sub region
-{
-	return $_[0]->{region} unless $#_;
-	my ( $self, $region ) = @_;
-	if ( $region && !UNIVERSAL::isa($region, "Prima::PS::PostScript::Region")) {
-		if ( UNIVERSAL::isa($region, 'Prima::Region')) {
-			$region = Prima::PS::PostScript::Region-> new_from_region( $region, $self );
-		} else {
-			warn "Region is not a Prima::PS::PostScript::Region";
-			return undef;
-		}
-	}
-	$self->{clipRect} = [0,0,0,0];
-	$self->{region} = $region;
-	$self-> change_transform;
-}
-
-package
-	Prima::PS::PostScript::Path;
-use base qw(Prima::PS::Drawable::Path);
-
-my %dict = (
-	newpath   => 'N',
-	lineto    => 'l',
-	moveto    => 'M',
-	curveto   => 'U',
-	stroke    => 'O',
-	closepath => 'X',
-	fill_alt  => 'E',
-	fill_wind => 'F',
-);
-
-sub dict { \%dict }
-
-sub set_current_point
-{
-	my ( $self, $x, $y ) = @_;
-	$self-> emit($x, $y, $self->{move_is_line} ? 'l' : 'M');
-	$self-> {move_is_line} = 1;
-}
-
-sub region
-{
-	my ($self, $mode) = @_;
-	my $path = join "\n", @{$self-> entries};
-	$path .= ' X' unless $path =~ /X$/;
-	return Prima::PS::PostScript::Region->new( $path );
-}
-
-package
-	Prima::PS::PostScript::Region;
-use base qw(Prima::PS::Drawable::Region);
-use Prima::PS::Format;
-
-sub other { UNIVERSAL::isa($_[0], "Prima::PS::PostScript::Region") ? $_[0] : () }
-
-sub equals
-{
-	my $self = shift;
-	my $other = other(shift) or return;
-	return $self->{path} eq $other->{path};
-}
-
-sub combine
-{
-	my $self = shift;
-	my $other = other(shift) or return;
-	$self->{path} .= "\n" . $other->apply_offset;
-}
-
-sub is_empty { shift->{path} !~ /[OF]/ }
-
-sub apply_offset
-{
-	my ($self, $canvas) = @_;
-	my $path = $self->{path};
-	my @offset = @{ $self->{offset} };
-	return $path if 0 == grep { $_ != 0 } @offset;
-
-	@offset = $canvas-> pixel2point(@offset) if $canvas;
-	float_inplace(@offset);
-	return "@offset T $path -$offset[0] -$offset[1] T";
-}
-
-sub apply_clip
-{
-	my ($self, $canvas) = @_;
-	return $self->apply_offset($canvas) . ' C';
-}
-
 1;
 
 __END__
@@ -1077,26 +914,14 @@ Prima::PS::PostScript -  PostScript interface to Prima::Drawable
 
 =head1 SYNOPSIS
 
-Recommended usage:
-
-	use Prima::PS::Printer;
-	my $x = Prima::PS::File-> new( file => 'out.ps');
-
-or
-
-	my $x = Prima::PS::FileHandle-> new( handle => \&STDOUT);
-
-Low-level:
-
+	use Prima;
 	use Prima::PS::PostScript;
+
 	my $x = Prima::PS::PostScript-> create( onSpool => sub {
 		open F, ">> ./test.ps";
 		print F $_[1];
 		close F;
 	});
-
-Printing:
-
 	die "error:$@" unless $x-> begin_doc;
 	$x-> font-> size( 30);
 	$x-> text_out( "hello!", 100, 100);
@@ -1105,24 +930,24 @@ Printing:
 
 =head1 DESCRIPTION
 
-Implements the Prima library interface to PostScript level 2 document language.
-The module is designed to be compliant with the Prima::Drawable interface.
-All properties' behavior is as same as Prima::Drawable's except those
+Realizes the Prima library interface to PostScript level 2 document language.
+The module is designed to be compliant with Prima::Drawable interface.
+All properties' behavior is as same as Prima::Drawable's, except those
 described below.
 
 =head2 Inherited properties
 
 =over
 
-=item resolution
+=item ::resolution
 
-Can be set while the object is in the normal stage - cannot be changed if the
-document is opened. Applies to implementation of the fillPattern property and
-general pixel-to-point and vice versa calculations
+Can be set while object is in normal stage - cannot be changed if document
+is opened. Applies to fillPattern realization and general pixel-to-point
+and vice versa calculations
 
-=item alpha
+=item ::region
 
-C<alpha> is not implemented
+- ::region is not realized ( yet?)
 
 =back
 
@@ -1130,27 +955,35 @@ C<alpha> is not implemented
 
 =over
 
-=item copies
+=item ::copies
 
-The number of copies that the PS interpreter should print
+amount of copies that PS interpreter should print
 
-=item grayscale
+=item ::grayscale
 
 could be 0 or 1
 
-=item pageSize
+=item ::pageSize
 
-The physical page dimension, in points
+physical page dimension, in points
 
-=item pageMargins
+=item ::pageMargins
 
-Non-printable page area, an array of 4 integers:
-left, bottom, right, and top margins in points.
+non-printable page area, an array of 4 integers:
+left, bottom, right and top margins in points.
 
-=item reversed
+=item ::reversed
 
 if 1, a 90 degrees rotated document layout is assumed
 
+=item ::rotate and ::scale
+
+along with Prima::Drawable::translate provide PS-specific
+transformation matrix manipulations. ::rotate is number,
+measured in degrees, counter-clockwise. ::scale is array of
+two numbers, respectively x- and y-scale. 1 is 100%, 2 is 200%
+etc.
+
 =back
 
 =head2 Internal methods
@@ -1166,25 +999,24 @@ Can be called for direct PostScript code injection. Example:
 
 =item pixel2point and point2pixel
 
-Helpers for translation from pixels to points and vice versa.
+Helpers for translation from pixel to points and vice versa.
 
 =item fill & stroke
 
-Wrappers for PS outline that are expected to be either filled or stroked.
-Apply colors, line, and fill styles if necessary.
+Wrappers for PS outline that is expected to be filled or stroked.
+Apply colors, line and fill styles if necessary.
 
 =item spool
 
-Prima::PS::PostScript is not responsible for the output of the generated
-document, it only calls the C<::spool> method when the document is closed
-through an C<::end_doc> call. By default discards the data.  The
-C<Prima::PS::Printer> class handles the spooling logic.
+Prima::PS::PostScript is not responsible for output of
+generated document, it just calls ::spool when document
+is closed through ::end_doc. By default just skips data.
+Prima::PS::Printer handles spooling logic.
 
 =item fonts
 
-Calls Prima::Application::fonts and returns its output filtered so that only
-the fonts that support the C<iso10646-1> encoding are present. This effectively
-allows only unicode output.
+Returns Prima::Application::fonts, however with C<iso10646-1> encoding only.
+That effectively allows only unicode output.
 
 =back
 
