@@ -1,6 +1,6 @@
 #
 #  Created by:
-#     Dmitry Karasik <dk@plab.ku.dk> 
+#     Dmitry Karasik <dk@plab.ku.dk>
 #  Modifications by:
 #     Anton Berezin  <tobez@tobez.org>
 #  Documentation by:
@@ -23,6 +23,7 @@ sub profile_default
 	return {
 		%{$_[ 0]-> SUPER::profile_default},
 		autoTrack     => 1,
+		dndAware      => 1,
 		growMode      => gm::GrowHiX,
 		height        => $stdMetrics[1],
 		min           => 0,
@@ -125,7 +126,7 @@ sub draw_pad
 	}
 
 	$self-> rect_bevel( $canvas, @{$self-> {$part}-> {rect}},
-		width   => 2, 
+		width   => 2,
 		concave => $self->{$part}->{pressed},
 		fill    => $canvas->new_gradient(
 			palette   => \@palette,
@@ -174,7 +175,10 @@ sub on_paint
 	$canvas-> color( $c3d[1]);
 	$canvas-> line( 0, $maxy, $maxx, $maxy);
 	$canvas-> line( 0, 0, 0, $maxy);
-	{
+
+	my $collapsed = ($v ? $maxy : $maxx) < $self->{minThumbSize};
+
+	unless ( $collapsed ) {
 		$self-> draw_pad( $canvas, b1 => $clr[1]);
 		$canvas-> color( $self-> {b1}-> { enabled} ? $clr[ 0] : $self-> disabledColor);
 		my $a = $self-> { b1}-> { pressed} ? 1 : 0;
@@ -191,7 +195,8 @@ sub on_paint
 		$canvas-> fillpoly( [ @spot])
 			if $maxx > 10 && $maxy > 10;
 	}
-	{
+
+	unless ( $collapsed ) {
 		$self-> draw_pad( $canvas, b2 => $clr[1]);
 		$canvas-> color( $self-> {b2}-> { enabled} ? $clr[ 0] : $self-> disabledColor);
 		my $a = $self-> { b2}-> { pressed} ? 1 : 0;
@@ -209,6 +214,7 @@ sub on_paint
 			if $maxx > 10 && $maxy > 10;
 		$canvas-> color( $clr[ 1]);
 	}
+
 	if ( $self-> { tab}-> { enabled})
 	{
 		my @rect = @{$self-> { tab}-> { rect}};
@@ -235,9 +241,9 @@ sub on_paint
 		}
 
 		$self-> draw_pad( $canvas, tab => $clr[1]);
-		if ( 
-			$self-> {minThumbSize} > 8 && 
-			$self->{style} ne 'xp' && 
+		if (
+			$self-> {minThumbSize} > 8 &&
+			$self->{style} ne 'xp' &&
 			(( $v ? $maxx : $maxy) > 10)
 		)
 		{
@@ -245,9 +251,9 @@ sub on_paint
 			$canvas-> backColor( $c3d[ 0]);
 			if ( $v)
 			{
-				my $sty = $rect[1] + 
-					int($lenx / 2) - 
-					4 - 
+				my $sty = $rect[1] +
+					int($lenx / 2) -
+					4 -
 					( $self-> { tab} -> { pressed} ? 1 : 0);
 				my $stx = int($maxx / 3) + ( $self-> { tab} -> { pressed} ? 1 : 0);
 				my $lnx = int($maxx / 3);
@@ -256,9 +262,9 @@ sub on_paint
 				$canvas-> fillPatternOffset($stx, $sty);
 				$canvas-> bar( $stx, $sty - 1, $stx + $lnx, $sty + 9);
 			} else {
-				my $stx = $rect[0] + 
-					int($lenx / 2) - 
-					4 + 
+				my $stx = $rect[0] +
+					int($lenx / 2) -
+					4 +
 					( $self-> { tab}-> { pressed} ? 1 : 0) ;
 				my $sty = int($maxy / 3) - ( $self-> { tab} -> { pressed} ? 1 : 0);
 				my $lny = int($maxy / 3);
@@ -269,9 +275,11 @@ sub on_paint
 			}
 		}
 	} else {
-		my @r = @{$self-> {groove}-> {rect}};
+		my @r = $collapsed ? 
+			( 0, 0, $maxx, $maxy ) :
+			@{$self-> {groove}-> {rect}};
 		$canvas-> color( $c3d[1]);
-		$v ? 
+		$v ?
 			( $canvas-> line( $r[0]-1, $r[1], $r[0]-1, $r[3])):
 			( $canvas-> line( $r[0], $r[3]+1, $r[2], $r[3]+1));
 		$canvas-> color( $clr[1]);
@@ -290,6 +298,7 @@ sub on_paint
 sub translate_point
 {
 	my ( $self, $x, $y) = @_;
+	return undef if ($self->vertical ? $self->height : $self->width) < $self->{minThumbSize};
 	for( qw(b1 b2 tab left right groove))
 	{
 		next unless defined $self-> {$_}-> {rect};
@@ -390,11 +399,12 @@ sub on_mousedown
 	return if defined $self-> {mouseTransaction};
 	my $who = $self-> translate_point( $x, $y);
 	return if !defined $who or $who eq q(groove);
+	return if $self->{drop_transaction} and $who !~ /^b\d$/;
 	if (( $who eq q(b1) || $who eq q(b2))) {
 		if ( $self-> {$who}-> {enabled}) {
 			$self-> {$who}-> {pressed} = 1;
 			$self-> {mouseTransaction} = $who;
-			$self-> capture(1);
+			$self-> capture(1) unless $self->{drop_transaction};
 			$self-> {suppressNotify} = 1;
 			$self-> value( $self-> {value} + (( $who eq q(b1))?-1:1) * $self-> {step});
 			$self-> {suppressNotify} = undef;
@@ -410,7 +420,7 @@ sub on_mousedown
 		$self-> {$who}-> {pressed} = 1;
 		$self-> {mouseTransaction} = $who;
 		$self-> value( $self-> {value} + (( $who eq q(left))?-1:1) * $self-> {pageStep});
-		$self-> capture(1);
+		$self-> capture(1) unless $self->{drop_transaction};
 		$self-> scroll_timer_start;
 		return;
 	}
@@ -420,7 +430,7 @@ sub on_mousedown
 		my @r = @{$self-> {tab}-> {rect}};
 		$self-> {$who}-> {aperture} = $self-> { vertical} ? $y - $r[1] : $x - $r[0];
 		$self-> draw_part($who);
-		$self-> capture(1);
+		$self-> capture(1) unless $self->{drop_transaction};
 	}
 }
 
@@ -439,16 +449,17 @@ sub on_mouseup
 	return unless defined $self-> {mouseTransaction};
 
 	my $who = $self-> {mouseTransaction};
-	if ( 
-		$who eq q(b1) || 
-		$who eq q(b2) || 
-		$who eq q(left) || 
-		$who eq q(right) || 
+	return if $self->{drop_transaction} and $who !~ /^b\d$/;
+	if (
+		$who eq q(b1) ||
+		$who eq q(b2) ||
+		$who eq q(left) ||
+		$who eq q(right) ||
 		$who eq q(tab)
 	) {
 		$self-> {$who}-> {pressed} = 0;
 		$self-> {mouseTransaction} = undef;
-		$self-> capture(0);
+		$self-> capture(0) unless $self->{drop_transaction};
 		$self-> repaint;
 		$self-> notify(q(Change)) if !$self-> {autoTrack} && $who eq q(tab);
 	}
@@ -475,6 +486,7 @@ sub on_mousemove
 		return;
 	}
 
+	return if $self->{drop_transaction} and $who !~ /^b\d$/;
 	if ( $who eq q(tab)) {
 		my @groove = @{$self-> {groove}-> {rect}};
 		my @tab    = @{$self-> {tab}-> {rect}};
@@ -495,10 +507,10 @@ sub on_mousemove
 			$self-> {suppressNotify} = undef;
 			$self-> notify(q(Track)) if !$self-> {autoTrack} && $ov != $self-> {value};
 		}
-	} elsif ( 
-		$who eq q(b1) || 
-		$who eq q(b2) || 
-		$who eq q(left) || 
+	} elsif (
+		$who eq q(b1) ||
+		$who eq q(b2) ||
+		$who eq q(left) ||
 		$who eq q(right)
 	) {
 		my $upon  = $self-> translate_point( $x, $y);
@@ -518,36 +530,63 @@ sub on_mouseleave
 sub on_mousewheel
 {
 	my ( $self, $mod, $x, $y, $z) = @_;
-	$self-> value( $self-> value - $self-> step * int( $z/120));
+	$z = (abs($z) > 120) ? int($z/120) : (($z > 0) ? 1 : -1);
+	$self-> value( $self-> value - $self-> step * $z);
 	$self-> clear_event;
 }
 
+sub on_dragbegin
+{
+	my $self = shift;
+	$self->{drop_transaction} = [0,0];
+}
+
+sub on_dragover
+{
+	my ($self, $clipboard, $action, $mod, $x, $y, $ref) = @_;
+	$ref->{allow} = 0;
+	if ( $self-> {mouseTransaction} ) {
+		$self->notify(q(MouseMove), 0, $x, $y);
+		@{$self->{drop_transaction}} = [$x, $y];
+	} else {
+		$self->notify(q(MouseDown), mb::Left, 0, $x, $y);
+	}
+}
+
+sub on_dragend
+{
+	my ($self, $clipboard, $action, $mod, $x, $y, $ref) = @_;
+	$ref->{allow} = 0;
+	$self->notify(q(MouseUp), mb::Left, 0, $x, $y);
+	undef $self->{drop_transaction};
+}
+
 sub reset
 {
 	my $self = $_[0];
-	
+
 	$self-> { b1} -> { enabled} = $self-> { value} > $self-> { min};
 	$self-> { b2} -> { enabled} = $self-> { value} < $self-> { max};
 	my $fullDisable = $self-> { partial} == $self-> { whole};
 	$self-> { tab}-> { enabled} = ( $self-> { min} != $self-> { max}) && !$fullDisable;
 	$self-> { b1}-> { enabled} = 0 if ( $self-> { value} == $self-> { min});
 	$self-> { b2}-> { enabled} = 0 if ( $self-> { value} == $self-> { max});
-	
+
 	my $btx  = $self-> { minThumbSize};
 	my $v    = $self-> { vertical};
 	my @size = $self-> size;
 	my ( $maxx, $maxy) = ( $size[0]-1, $size[1]-1);
-	
-	if ( $v) { 
-		$btx = int($size[1] / 2) if $btx * 2 >= $maxy; 
-	} else { 
-		$btx = int($size[0] / 2) if $btx * 2 >= $maxx; 
+
+	if ( $v) {
+		$btx = int($size[1] / 2) if $btx * 2 >= $maxy;
+	} else {
+		$btx = int($size[0] / 2) if $btx * 2 >= $maxx;
 	}
-	my @rect = $v ? 
-		( 1, $maxy - $btx + 1, $maxx - 1, $maxy - 1) : 
+	my @rect = $v ?
+		( 1, $maxy - $btx + 1, $maxx - 1, $maxy - 1) :
 		( 1, 1, $btx - 1, $maxy - 1);
 	$self-> { b1}-> { rect} = [ @rect];
-	@rect  = $v ? 
+	@rect  = $v ?
 		( 1, 1, $maxx - 1, $btx - 1) :
 		( $maxx - $btx + 1, 1, $maxx - 1, $maxy - 1);
 	$self-> { b2}-> { rect} = [ @rect];
@@ -563,13 +602,13 @@ sub reset
 	my $startx  = $v ? $size[1]: $size[0];
 	my $groovex = $startx - $btx * 2;
 	$self-> { tab}-> { enabled} = 0 if $groovex < $self-> {minThumbSize};
-	
+
 	if ( $self-> { tab}-> { enabled}) {
 		my $lenx = int( $groovex * $self-> { partial} / $self-> { whole});
 		$lenx = $self-> {minThumbSize} if $lenx < $self-> {minThumbSize};
-		my $atx  = 
-			int(( $self-> { value} - $self-> {min}) * 
-			( $groovex - $lenx) / 
+		my $atx  =
+			int(( $self-> { value} - $self-> {min}) *
+			( $groovex - $lenx) /
 			( $self-> { max} - $self-> { min}));
 		$atx = $groovex - $lenx if $lenx + $atx > $groovex;
 		( $lenx, $atx) = ( $groovex - 1, 0) if $atx < 0;
@@ -585,7 +624,7 @@ sub reset
 		);
 		$self-> { tab}-> { rect} = [@rect];
 		$self-> { tab}-> { q(length)} = $lenx;
-		
+
 		if ( $v) {
 			my @r2 = ( 2, $btx, $maxx - 1, $rect[1] - 1);
 			my @r1 = ( 2, $rect[3], $maxx - 1, $maxy - $btx);
@@ -606,7 +645,7 @@ sub set_bounds
 	my ( $self, $min, $max) = @_;
 	$max = $min if $max < $min;
 	( $self-> { min}, $self-> { max}) = ( $min, $max);
-	
+
 	my $oldValue = $self-> {value};
 	$self-> value( $max) if $self-> {value} > $max;
 	$self-> value( $min) if $self-> {value} < $min;
@@ -769,7 +808,7 @@ C-implemented parts except those inherited from C<Prima::Widget>.
 
 =head2 Properties
 
-=over 
+=over
 
 =item autoTrack BOOLEAN
 
@@ -808,7 +847,7 @@ Default value: 0
 
 =item minThumbSize INTEGER
 
-A minimal thumb breadth in pixels. The thumb cannot have 
+A minimal thumb breadth in pixels. The thumb cannot have
 main dimension lesser than this.
 
 Default value: 21
@@ -879,8 +918,8 @@ Default value is 100.
 
 =item get_default_size
 
-Returns two integers, the default platform dependant width 
-of a vertical scrollbar and height of a horizontal scrollbar.  
+Returns two integers, the default platform dependant width
+of a vertical scrollbar and height of a horizontal scrollbar.
 
 =back
 
