@@ -62,7 +62,7 @@ sub on_paint
 	my $cap = $self-> text;
 	unless ( defined $self-> {image}) {
 		$canvas-> rect3d( 0, 0, $size[0]-1, $size[1]-1, $bw, @c3d, $clr);
-		$canvas-> text_out_bidi( $cap,
+		$canvas-> text_shape_out( $cap,
 			( $size[0] - $canvas-> get_text_width( $cap)) / 2,
 			( $size[1] - $canvas-> font-> height) / 2,
 		) if defined $cap;
@@ -81,7 +81,7 @@ sub on_paint
 		}
 		$x += $dx;
 	}
-	$canvas-> text_out_bidi( $cap,
+	$canvas-> text_shape_out( $cap,
 		( $size[0] - $canvas-> get_text_width( $cap)) / 2,
 		( $size[1] - $canvas-> font-> height) / 2,
 	) if $cap;
@@ -155,8 +155,6 @@ sub raise        {($#_)?$_[0]-> set_raise($_[1]):return $_[0]-> {raise}}
 
 1;
 
-__DATA__
-
 =pod
 
 =head1 NAME
@@ -204,8 +202,8 @@ Default value: 1
 
 =item zoom INTEGER
 
-Selects zoom level for image display. 
-The acceptable value range is between 1 and 10. 
+Selects zoom level for image display.
+The acceptable value range is between 1 and 10.
 
 Default value: 1
 
