@@ -1,6 +1,6 @@
 package fontdlg;
 
-=pod 
+=pod
 
 =head1 NAME
 
@@ -11,7 +11,7 @@ examples/fontdlg.pl - An alternate font selection window
 Demonstrates Prima font API and its usage.
 
 Note the inability to set a font with a particular size and width factor in one
-call ( in $re_sample sub ). A font size and width is accepted, however. 
+call ( in $re_sample sub ). A font size and width is accepted, however.
 
 Tests the Prima font interface implementation. A constant pain here is the
 correspondence of a font metrics before and after the font load.  X is known
@@ -28,7 +28,7 @@ use warnings;
 use Carp;
 use Prima;
 use Prima::Application name => "Font Dialog";
-use Prima qw(Lists Sliders Buttons FileDialog);
+use Prima qw(Lists Sliders Buttons Dialog::FileDialog);
 use Encode;
 
 # try to use perl5.8 glyph names
@@ -46,11 +46,262 @@ my $displayRes;
 my $fs = 0;
 my $fd = 0;
 my $fpitch = fp::Default;
+my $fvector = fv::Default;
 my $fwidth = 0;
 
+our %languages = (
+	'aa'     => q(Afar),
+	'ab'     => q(Abkhazia),
+	'af'     => q(Afrikaans),
+	'ak'     => q(Akan),
+	'am'     => q(Amharic),
+	'an'     => q(Aragonese),
+	'ar'     => q(Arabic),
+	'as'     => q(Assamese),
+	'ast'    => q(Asturian),
+	'av'     => q(Avaric),
+	'ay'     => q(Aymara),
+	'az-az'  => q(Azerbaijani in Azerbaijan),
+	'az-ir'  => q(Azerbaijani in Iran),
+	'ba'     => q(Bashkir),
+	'be'     => q(Byelorussian),
+	'ber-dz' => q(Berber in Algeria),
+	'ber-ma' => q(Berber in Morocco),
+	'bg'     => q(Bulgarian),
+	'bh'     => q(Bihari),
+	'bho'    => q(Bhojpuri),
+	'bi'     => q(Bislama),
+	'bin'    => q(Edo or Bini),
+	'bm'     => q(Bambara),
+	'bn'     => q(Bengali),
+	'bo'     => q(Tibetan),
+	'br'     => q(Breton),
+	'brx'    => q(Bodo),
+	'bs'     => q(Bosnian),
+	'bua'    => q(Buriat),
+	'byn'    => q(Blin),
+	'ca'     => q(Catalan),
+	'ce'     => q(Chechen),
+	'ch'     => q(Chamorro),
+	'chm'    => q(Mari),
+	'chr'    => q(Cherokee),
+	'co'     => q(Corsican),
+	'crh'    => q(Crimean Tatar),
+	'cs'     => q(Czech),
+	'csb'    => q(Kashubian),
+	'cu'     => q(Old Church Slavonic),
+	'cv'     => q(Chuvash),
+	'cy'     => q(Welsh),
+	'da'     => q(Danish),
+	'de'     => q(German),
+	'doi'    => q(Dogri),
+	'dv'     => q(Divehi),
+	'dz'     => q(Dzongkha),
+	'ee'     => q(Ewe),
+	'el'     => q(Greek),
+	'en'     => q(English),
+	'eo'     => q(Esperanto),
+	'es'     => q(Spanish),
+	'et'     => q(Estonian),
+	'eu'     => q(Basque),
+	'fa'     => q(Persian),
+	'fat'    => q(Fanti),
+	'ff'     => q(Fulah),
+	'fi'     => q(Finnish),
+	'fil'    => q(Filipino),
+	'fj'     => q(Fijian),
+	'fo'     => q(Faroese),
+	'fr'     => q(French),
+	'fur'    => q(Friulian),
+	'fy'     => q(Frisian),
+	'ga'     => q(Irish),
+	'gd'     => q(Scots Gaelic),
+	'gez'    => q(Ethiopic),
+	'gl'     => q(Galician),
+	'gn'     => q(Guaraní),
+	'gu'     => q(Gujarati),
+	'gv'     => q(Manx Gaelic),
+	'ha'     => q(Hausa),
+	'haw'    => q(Hawaiian),
+	'he'     => q(Hebrew),
+	'hi'     => q(Hindi),
+	'hne'    => q(Chhattisgarhi),
+	'ho'     => q(Hiri Motu),
+	'hr'     => q(Croatian),
+	'hsb'    => q(Upper Sorbian),
+	'ht'     => q(Haitian),
+	'hu'     => q(Hungarian),
+	'hy'     => q(Armenian),
+	'hz'     => q(Herero),
+	'ia'     => q(Interlingua),
+	'id'     => q(Indonesian),
+	'ie'     => q(Interlingue),
+	'ig'     => q(Igbo),
+	'ii'     => q(Sichuan Yi),
+	'ik'     => q(Inupiaq),
+	'io'     => q(Ido),
+	'is'     => q(Icelandic),
+	'it'     => q(Italian),
+	'iu'     => q(Inuktitut),
+	'ja'     => q(Coverage from JIS X 0208),
+	'jv'     => q(Javanese),
+	'ka'     => q(Georgian),
+	'kaa'    => q(Kara-Kalpak),
+	'kab'    => q(Kabyle),
+	'ki'     => q(Kikuyu),
+	'kj'     => q(Kuanyama),
+	'kk'     => q(Kazakh),
+	'kl'     => q(Greenlandic),
+	'km'     => q(Central Khmer),
+	'kn'     => q(Kannada),
+	'ko'     => q(Korean),
+	'kok'    => q(Kokani),
+	'kr'     => q(Kanuri),
+	'ks'     => q(Kashmiri),
+	'ku-am'  => q(Kurdish in Armenia),
+	'ku-iq'  => q(Kurdish in Iraq),
+	'ku-ir'  => q(Kurdish in Iran),
+	'ku-tr'  => q(Kurdish in Turkey),
+	'kum'    => q(Kumyk),
+	'kv'     => q(Komi),
+	'kw'     => q(Cornish),
+	'kwm'    => q(Kwambi),
+	'ky'     => q(Kirgiz),
+	'la'     => q(Latin),
+	'lah'    => q(Lahnda),
+	'lb'     => q(Luxembourgish),
+	'lez'    => q(Lezghian),
+	'lg'     => q(Ganda),
+	'li'     => q(Limburgan),
+	'ln'     => q(Lingala),
+	'lo'     => q(Lao),
+	'lt'     => q(Lithuanian),
+	'lv'     => q(Latvian),
+	'mai'    => q(Maithili),
+	'mg'     => q(Malagasy),
+	'mh'     => q(Marshallese),
+	'mi'     => q(Maori),
+	'mk'     => q(Macedonian),
+	'ml'     => q(Malayalam),
+	'mn-cn'  => q(Mongolian in China),
+	'mn-mn'  => q(Mongolian in Mongolia),
+	'mni'    => q(Maniputi),
+	'mo'     => q(Moldavian),
+	'mr'     => q(Marathi),
+	'ms'     => q(Malay),
+	'mt'     => q(Maltese),
+	'my'     => q(Burmese),
+	'na'     => q(Nauru),
+	'nb'     => q(Norwegian Bokmål),
+	'nds'    => q(Low Saxon),
+	'ne'     => q(Nepali),
+	'ng'     => q(Ndonga),
+	'nl'     => q(Dutch),
+	'nn'     => q(Norwegian Nynorsk),
+	'no'     => q(Norwegian),
+	'nqo'    => q(N'Ko),
+	'nr'     => q(Ndebele, South),
+	'nso'    => q(Northern Sotho),
+	'nv'     => q(Navajo),
+	'ny'     => q(Chichewa),
+	'oc'     => q(Occitan),
+	'om'     => q(Oromo or Galla),
+	'or'     => q(Oriya),
+	'os'     => q(Ossetic),
+	'ota'    => q(Ottoman Turkish),
+	'pa'     => q(Panjabi),
+	'pa-pk'  => q(Panjabi),
+	'pap-an' => q(Papiamento in Netherlands Antilles),
+	'pap-aw' => q(Papiamento in Aruba),
+	'pes'    => q(Western Farsi),
+	'pl'     => q(Polish),
+	'prs'    => q(Dari),
+	'ps-af'  => q(Pashto in Afghanistan),
+	'ps-pk'  => q(Pashto in Pakistan),
+	'pt'     => q(Portuguese),
+	'qu'     => q(Quechua),
+	'quz'    => q(Cusco Quechua),
+	'rm'     => q(Rhaeto-Romance),
+	'rn'     => q(Rundi),
+	'ro'     => q(Romanian),
+	'ru'     => q(Russian),
+	'rw'     => q(Kinyarwanda),
+	'sa'     => q(Sanskrit),
+	'sah'    => q(Yakut),
+	'sat'    => q(Santali),
+	'sc'     => q(Sardinian),
+	'sco'    => q(Scots),
+	'sd'     => q(Sindhi),
+	'se'     => q(North Sámi),
+	'sel'    => q(Selkup),
+	'sg'     => q(Sango),
+	'sh'     => q(Serbo-Croatian),
+	'shs'    => q(Secwepemctsin),
+	'si'     => q(Sinhala),
+	'sid'    => q(Sidamo),
+	'sk'     => q(Slovak),
+	'sl'     => q(Slovenian),
+	'sm'     => q(Samoan),
+	'sma'    => q(South Sámi),
+	'smj'    => q(Lule Sámi),
+	'smn'    => q(Inari Sámi),
+	'sms'    => q(Skolt Sámi),
+	'sn'     => q(Shona),
+	'so'     => q(Somali),
+	'sq'     => q(Albanian),
+	'sr'     => q(Serbian),
+	'ss'     => q(Swati),
+	'st'     => q(Sotho, Southern),
+	'su'     => q(Sundanese),
+	'sv'     => q(Swedish),
+	'sw'     => q(Swahili),
+	'syr'    => q(Syriac),
+	'ta'     => q(Tamil),
+	'te'     => q(Telugu),
+	'tg'     => q(Tajik),
+	'th'     => q(Thai),
+	'ti-er'  => q(Eritrean Tigrinya),
+	'ti-et'  => q(Ethiopian Tigrinya),
+	'tig'    => q(Tigre),
+	'tk'     => q(Turkmen),
+	'tl'     => q(Tagalog),
+	'tn'     => q(Tswana),
+	'to'     => q(Tonga),
+	'tr'     => q(Turkish),
+	'ts'     => q(Tsonga),
+	'tt'     => q(Tatar),
+	'tw'     => q(Twi),
+	'ty'     => q(Tahitian),
+	'tyv'    => q(Tuvinian),
+	'ug'     => q(Uyghur),
+	'uk'     => q(Ukrainian),
+	'ur'     => q(Urdu),
+	'uz'     => q(Uzbek),
+	've'     => q(Venda),
+	'vi'     => q(Vietnamese),
+	'vo'     => q(Volapük),
+	'vot'    => q(Votic),
+	'wa'     => q(Walloon),
+	'wal'    => q(Wolaitta),
+	'wen'    => q(Sorbian languages),
+	'wo'     => q(Wolof),
+	'xh'     => q(Xhosa),
+	'yap'    => q(Yapese),
+	'yi'     => q(Yiddish),
+	'yo'     => q(Yoruba),
+	'za'     => q(Zhuang),
+	'zh-cn'  => q(Chinese),
+	'zh-hk'  => q(Chinese Hong Kong Supplementary Character Set),
+	'zh-mo'  => q(Chinese in Macau),
+	'zh-sg'  => q(Chinese in Singapore),
+	'zh-tw'  => q(Chinese),
+	'zu'     => q(Zulu),
+);
+
 my $re_sample = sub {
 	return if $w-> {exampleFontSet};
-	my $fn = $fontList{ $w-> NameList-> get_item_text($w-> NameList-> focusedItem)}{name};
+	my $fid = $w-> NameList-> get_item_text( $w-> NameList-> focusedItem);
+	my $fn = ($fid =~ /not defined/) ? 'Default' : $fontList{$fid}{name};
 	$w-> {exampleFontSet} = 1;
 	my $i = $w-> SizeList-> focusedItem;
 	my $enc = $w-> Encoding-> get_item_text( $w-> Encoding-> focusedItem);
@@ -63,6 +314,7 @@ my $re_sample = sub {
 		style       => $fs,
 		direction   => $fd,
 		pitch       => $fpitch,
+		vector      => $fvector,
 		encoding    => $enc,
 	);
 
@@ -84,15 +336,18 @@ my $lastEncSel = "";
 
 my $re_size = sub {
 	my $name_changed = $_[0];
-	my $fn = $fontList{ $w-> NameList-> get_item_text( $w-> NameList-> focusedItem)}{name};
+	my $fid = $w-> NameList-> get_item_text( $w-> NameList-> focusedItem);
+	my $fn = ($fid =~ /not defined/) ? undef : $fontList{$fid}{name};
 	my @sizes = ();
 	my $current_encoding = ( $lastEncSel eq '(any)' || $name_changed) ? '' : $lastEncSel;
-	my @list = @{$::application-> fonts( $fn, $name_changed ? '' : $current_encoding)};
+	my @list = $fn ? 
+		@{$::application-> fonts( $fn, $name_changed ? '' : $current_encoding)} :
+		({vector => 1});
 
 	if ( $name_changed) {
 		my %enc;
 		my @enc_items;
-		for ( map { $_-> {encoding}} @list) {
+		for ( grep { defined } map { $_-> {encoding}} @list) {
 			next if $enc{$_};
 			push ( @enc_items, $_ );
 			$enc{$_} = 1;
@@ -107,7 +362,7 @@ my $re_size = sub {
 		$w-> Encoding-> set_items( \@enc_items);
 		$w-> Encoding-> set_focused_item( $found);
 	}
-	
+
 	for ( @list)
 	{
 		next if length( $current_encoding) && ( $current_encoding ne $_-> {encoding});
@@ -118,7 +373,7 @@ my $re_size = sub {
 		} else {
 			push ( @sizes, $_-> {size});
 		}
-	}   
+	}
 	my %k = map { $_ => 1 } @sizes;
 	@sizes = sort { $a <=> $b } keys %k;
 	@sizes = (10) unless scalar @sizes;
@@ -147,7 +402,7 @@ my $re_size = sub {
 
 $w = Prima::MainWindow-> create( text => "Font Window",
 	origin => [ 200, 200],
-	size   => [ 500, 530],
+	size   => [ 500, 590],
 	designScale => [7,16],
 	borderStyle => bs::Dialog,
 );
@@ -167,7 +422,7 @@ sub create_info_window
 			$p-> font-> direction(0);
 
 			my $m = $p-> get_font;
-			my $xtext = Encode::decode('latin1', "\x{c5}Mg"); 
+			my $xtext = Encode::decode('latin1', "\x{c5}Mg");
 			my $s = $size[1] - $m-> {height} - $m-> {externalLeading} - 20;
 			my $w = $p-> get_text_width($xtext) + 66;
 			$p-> textOutBaseline(1);
@@ -220,7 +475,6 @@ sub create_info_window
 			$p-> linePattern( lp::Dash);
 			$p-> line( 2, $sd, $w, $sd);
 
-
 			$p-> font-> set(
 				height => 16,
 				pitch  => fp::Fixed,
@@ -236,6 +490,7 @@ sub create_info_window
 			$p-> text_out( 'internal leading    : '.$m-> {internalLeading}, 2, $sd); $sd -= $fh;
 			$p-> text_out( 'external leading    : '.$m-> {externalLeading}, 2, $sd); $sd -= $fh;
 			$p-> text_out( 'maximal width       : '.$m-> {maximalWidth}, 2, $sd); $sd -= $fh;
+			$p-> text_out( 'vector              : '.$m-> {vector}, 2, $sd); $sd -= $fh;
 			$p-> text_out( 'horizontal dev.res. : '.$m-> {xDeviceRes}, 2, $sd); $sd -= $fh;
 			$p-> text_out( 'vertical dev.res.   : '.$m-> {yDeviceRes}, 2, $sd); $sd -= $fh;
 			$p-> text_out( 'first char          : '.$m-> {firstChar}, 2, $sd); $sd -= $fh;
@@ -244,12 +499,33 @@ sub create_info_window
 			$p-> text_out( 'default char        : '.$m-> {defaultChar}, 2, $sd); $sd -= $fh;
 			$p-> text_out( 'family              : '.$m-> {family   }, 2, $sd); $sd -= $fh;
 			$p-> text_out( 'face name           : '.$cachedFacename, 2, $sd); $sd -= $fh;
+			unless ( $p-> Languages-> visible ) {
+				my $l = $p-> Languages;
+				my $em = $p->get_text_width('m');
+				$l->origin( 2 + $em * 22, $sd );
+				$l->font( $p-> font );
+				$l->size( $em * 30, $fh);
+				$l->show;
+			}
+			$p-> text_out( 'languages           : ', 2, $sd); $sd -= $fh;
 		},
 	);
 
+	$ww-> insert( ComboBox => 
+		name     => 'Languages',
+		style    => cs::DropDownList,
+		items    => [ 
+			sort map { $languages{$_} // $_ } 
+			@{$ww->get_font_languages // ['<none>']} 
+		],
+		growMode => gm::GrowLoY,
+		visible  => 0,
+		text     => '',
+	);
+
 	my %wf = ( %$f, direction => 0 );
 	my @gsize= ( $f->maximalWidth + 10 , $f->height + 10 );
-	my $glyph = $ww-> insert( Widget => 
+	my $glyph = $ww-> insert( Widget =>
 		text => ' ',
 		font => \%wf,
 		size => \@gsize,
@@ -258,7 +534,7 @@ sub create_info_window
 		backColor => cl::White,
 		#buffered => 1,
 		name    => 'Glyph',
-		textOutBaseline => 0,
+		textOutBaseline => 1,
 		onPaint => sub {
 			my $self = shift;
 			$self-> clear;
@@ -266,7 +542,7 @@ sub create_info_window
 
 			my ( $a, $b, $c ) = @{ $self->get_font_abc( ord($C), ord($C), utf8::is_utf8($C)) };
 			my ( $d, $e, $f ) = @{ $self->get_font_def( ord($C), ord($C), utf8::is_utf8($C)) };
-			
+
 			my $w = (( $a < 0 ) ? 0 : $a) + $b + (( $c < 0 ) ? 0 : $c);
 			my $h = (( $d < 0 ) ? 0 : $d) + $e + (( $f < 0 ) ? 0 : $f);
 			$w = ( $self-> width  - $w ) / 2;
@@ -279,7 +555,7 @@ sub create_info_window
 			$dy -= $d if $d < 0;
 
 			my $fh = $self-> font->height;
-			$self-> text_out( $C, $dx, $dy );
+			$self-> text_out( $C, $dx, $dy + $self->font-> descent);
 
 			$dx = abs($a);
 			$dy = abs($d);
@@ -292,6 +568,13 @@ sub create_info_window
 			$self-> line(0, $dy, $self->width, $dy);
 			$dy = (( $d < 0 ) ? 0 : $d) + $e + (( $f < 0 ) ? 0 : $f) - abs($f);
 			$self-> line(0, $dy, $self->width, $dy);
+
+			$self-> linePattern(lp::Solid);
+			$self-> color(cl::LightRed);
+			my $path = $self-> new_path;
+			$path->translate( -$a, 0 ) if $a < 0;
+			$path-> text( $C, baseline => 0 );
+			$path->stroke;
 		},
 	);
 
@@ -309,7 +592,7 @@ sub create_info_window
 		$charmap{$count++} = $_ for $$_[0] .. $$_[1];
 	}
 	my $ih = int($f-> height * 1.5);
-	my $l = $ww-> insert( AbstractListViewer => 
+	my $l = $ww-> insert( AbstractListViewer =>
 		origin => [0,0],
 		size   => [$ww-> width, $ww-> height - $f-> height - $f-> externalLeading - $f-> descent - 360],
 		growMode => gm::Client,
@@ -353,13 +636,14 @@ sub create_info_window
 	);
 	$l-> count( $count);
 	$ww-> select;
+	return $ww;
 }
 
 $displayRes = ($w-> resolution)[1];
 
 my $load_fonts = sub {
 	%fontList = ();
-	@fontItems = ();
+	@fontItems = ("<not defined>");
 	for ( sort { $a-> {name} cmp $b-> {name}} @{$::application-> fonts})
 	{
 		$fontList{$_-> {name}} = $_;
@@ -390,7 +674,7 @@ $w-> insert( ListBox =>
 	},
 );
 
-$w-> insert( ListBox => 
+$w-> insert( ListBox =>
 	origin      => [ 270, 160],
 	size        => [ 200, 55],
 	name        => 'Encoding',
@@ -399,7 +683,7 @@ $w-> insert( ListBox =>
 		&$re_size(0);
 		&$re_sample;
 	},
-	
+
 );
 
 my $scaling = $::application->font->height / ($w->designScale)[1];
@@ -475,7 +759,7 @@ $w-> insert( Button =>
 	name   => 'Load',
 	font   => { height => 20 * $scaling },
 	onClick   => sub {
-		my $d = Prima::FileDialog-> new;
+		my $d = Prima::Dialog::FileDialog-> new;
 		my $f = $d->execute;
 		return unless defined $f;
 		my $error = '';
@@ -484,7 +768,7 @@ $w-> insert( Button =>
 			$error =~ s/at examples.*//;
 			return Prima::message("Error loading font$error");
 		}
-	
+
 		$load_fonts->();
 		$w->NameList->items(\@fontItems);
 	}
@@ -513,7 +797,7 @@ $csl-> insert( Button =>
 );
 
 my $rg = $w-> insert( RadioGroup =>
-	origin      => [ 25, 460],
+	origin      => [ 25, 525],
 	size        => [ 445, 58],
 	name        => 'Pitch',
 );
@@ -539,6 +823,34 @@ $rg-> insert( Radio =>
 	onClick =>  sub { $fpitch = fp::Variable; &$re_sample; },
 );
 
+my $rg2 = $w-> insert( RadioGroup =>
+	origin      => [ 25, 460],
+	size        => [ 445, 58],
+	name        => 'Vector',
+);
+
+$rg2-> insert( Radio =>
+	name   => 'Default',
+	origin => [ 15, 5],
+	onClick =>  sub { $fvector = fv::Default;  &$re_sample; },
+	checked => 1,
+);
+
+$rg2-> insert( Radio =>
+	name   => 'Bitmap',
+	origin => [ 160, 5],
+	onClick =>  sub { $fvector = fv::Bitmap;  &$re_sample; },
+	font    => { style => fs::Bold, pitch => fp::Fixed}, # yes, fixed
+);
+
+$rg2-> insert( Radio =>
+	name   => 'Outline',
+	origin => [ 305, 5],
+	font    => { vector => fv::Outline, style => fs::Italic },
+	onClick =>  sub { $fvector = fv::Outline; &$re_sample; },
+);
+
+
 $w-> insert( Slider =>
 	name     => 'Stretcher',
 	origin   => [ 25, 382],
@@ -576,7 +888,7 @@ $w-> insert( Widget =>
 	size      => [ 200, 120],
 	backColor => cl::White,
 	onPaint   => sub {
-		my ($fore, $back, $x, $y) = 
+		my ($fore, $back, $x, $y) =
 			($_[0]-> color, $_[0]-> backColor, $_[1]-> width, $_[1]-> height);
 		$_[1]-> color( $back);
 		$_[1]-> bar( 0, 0, $x, $y);
@@ -590,7 +902,7 @@ $w-> insert( Widget =>
 		RANGES: for ( $i = 0; $i < @ranges; $i += 2) {
 			for ( $j = $ranges[$i]; $j < $ranges[$i+1]; $j++) {
 				$latin     .= chr($j) if $j > 45 && $j < 128;
-				$non_latin .= chr($j) if $j > 256 && ( !$use_charnames || (charnames::viacode($j) || '') !~ 
+				$non_latin .= chr($j) if $j > 256 && ( !$use_charnames || (charnames::viacode($j) || '') !~
 					/(space|punctuation|mark|accent|point|combining|modifier)/i);
 				last RANGES if length($latin) > 32 || length($non_latin) > 32;
 			}
@@ -602,8 +914,8 @@ $w-> insert( Widget =>
 		pop @box;
 		my $width = $_[1]-> get_text_width( $probe);
 		my ( $ox, $oy) = (( $x - $width) / 2, ( $y - $_[1]-> font-> height) / 2);
-		$box[$_] += $ox for 0,2,4,6; 
-		$box[$_] += $oy for 1,3,5,7; 
+		$box[$_] += $ox for 0,2,4,6;
+		$box[$_] += $oy for 1,3,5,7;
 		@box[4,5,6,7] = @box[6,7,4,5];
 		$_[1]-> color( cl::Yellow);
 		$_[1]-> fillpoly(\@box);
@@ -647,7 +959,7 @@ $w-> insert( Widget =>
 		$_[0]-> capture(0);
 		$_[0]-> {drag} = 0;
 		$_[0]-> pointer( cr::Default);
-		my $x = $::application-> get_widget_from_point( 
+		my $x = $::application-> get_widget_from_point(
 			$_[0]-> client_to_screen( $_[3], $_[4])
 		);
 		return unless $x;
@@ -657,6 +969,8 @@ $w-> insert( Widget =>
 
 &$re_size(1);
 &$re_sample;
+#$w->NameList->focusedItem($#fontItems);
+#create_info_window($w)->size(800, 800);
 }
 
 run;
